# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
# SPDX-License-Identifier: Apache-2.0
import webbrowser
from argparse import Namespace
from unittest import mock

import pytest

from awscli.customizations.exceptions import ConfigurationError
from awscli.customizations.login.login import LoginCommand

DEFAULT_ARGS = Namespace(remote=False)
DEFAULT_GLOBAL_ARGS = Namespace(
    region='us-east-1', endpoint_url=None, verify_ssl=None
)
SAMPLE_ID_TOKEN = (
    'eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzM4NCJ9.eyJpc3MiOiJodHRwczovL3NpZ25pbi5hd'
    '3MuYW1hem9uLmNvbS9zaWduaW4iLCJpYXQiOjE3NjAxMTU3NjQsImV4cCI6MTc2MDExNjk'
    '2NSwiYXVkIjoiYXJuOmF3czpzaWduaW46OjpjbGkvc2FtZS1kZXZpY2UiLCJzdWIiOiJhc'
    'm46YXdzOmlhbTo6MDEyMzQ1Njc4OTAxMjp1c2VyL0FkbWluIiwic2Vzc2lvbl9hcm4iOiJ'
    'hcm46YXdzOmlhbTo6MDEyMzQ1Njc4OTAxMjp1c2VyL0FkbWluIn0.HcleGdakodn9ZbCsR'
    'nsF_F2n5TmQD-OW9zc9oMU7DtNrXQwxzg4jO40N2BMgiTyW'
)


@pytest.fixture
def mock_token_loader():
    return mock.Mock()


@pytest.fixture
def mock_config_file_writer():
    return mock.Mock()


@pytest.fixture
def mock_session():
    def config_variables(key):
        if key == 'config_file':
            return 'configfile'
        return None

    mock_session = mock.Mock()
    mock_session.profile = 'profile-name'
    mock_session.available_profiles = ['profile-name']
    mock_session.get_config_variable.side_effect = config_variables
    mock_session.full_config = {'profiles': {'profile-name': {}}}
    mock_session._profile_map = {'profile-name': {}}

    return mock_session


@pytest.fixture
def mock_login_command(
    mock_session, mock_token_loader, mock_config_file_writer
):
    return LoginCommand(
        mock_session,
        mock_token_loader,
        mock_config_file_writer,
    )


@mock.patch('awscli.customizations.login.utils.get_base_sign_in_uri')
@mock.patch(
    'awscli.customizations.login.utils.SameDeviceLoginTokenFetcher.fetch_token'
)
def test_run_main_same_device_flow(
    mock_token_fetcher,
    mock_base_sign_in_uri,
    mock_login_command,
    mock_token_loader,
    mock_config_file_writer,
):
    mock_base_sign_in_uri.return_value = 'https://foo'
    mock_token_fetcher.return_value = (
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
        'arn:aws:iam::0123456789012:user/Admin',
    )

    mock_login_command._run_main(DEFAULT_ARGS, DEFAULT_GLOBAL_ARGS)

    mock_token_fetcher.assert_called_once()

    mock_token_loader.save_token.assert_called_once_with(
        'arn:aws:iam::0123456789012:user/Admin',
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
    )

    mock_config_file_writer.update_config.assert_called_once_with(
        {
            '__section__': "profile profile-name",
            'login_session': 'arn:aws:iam::0123456789012:user/Admin',
        },
        'configfile',
    )


@mock.patch('awscli.customizations.login.utils.get_base_sign_in_uri')
@mock.patch(
    'awscli.customizations.login.utils.CrossDeviceLoginTokenFetcher.fetch_token'
)
def test_run_main_cross_device_flow(
    mock_token_fetcher,
    mock_base_sign_in_uri,
    mock_login_command,
    mock_token_loader,
    mock_config_file_writer,
):
    # Set the --remote argument
    args = Namespace(**vars(DEFAULT_ARGS))
    args.remote = True

    mock_base_sign_in_uri.return_value = 'https://foo'
    mock_token_fetcher.return_value = (
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
        'arn:aws:iam::0123456789012:user/Admin',
    )

    mock_login_command._run_main(args, DEFAULT_GLOBAL_ARGS)

    mock_token_fetcher.assert_called_once()

    mock_token_loader.save_token.assert_called_once_with(
        'arn:aws:iam::0123456789012:user/Admin',
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
    )

    mock_config_file_writer.update_config.assert_called_once_with(
        {
            '__section__': "profile profile-name",
            'login_session': 'arn:aws:iam::0123456789012:user/Admin',
        },
        'configfile',
    )


@mock.patch('awscli.customizations.login.utils.get_base_sign_in_uri')
@mock.patch(
    'awscli.customizations.login.utils.SameDeviceLoginTokenFetcher.fetch_token'
)
def test_no_verify_ssl_on_signin_client(
    mock_token_fetcher,
    mock_base_sign_in_uri,
    mock_login_command,
    mock_token_loader,
    mock_config_file_writer,
):
    mock_base_sign_in_uri.return_value = 'https://foo'
    mock_token_fetcher.return_value = (
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
        'arn:aws:iam::0123456789012:user/Admin',
    )

    # Simulate setting --no-verify-ssl
    global_args = Namespace(**vars(DEFAULT_GLOBAL_ARGS))
    global_args.verify_ssl = False

    mock_login_command._run_main(DEFAULT_ARGS, global_args)

    # Assert that verify was set correctly on the custom signin client
    mock_login_command._session.create_client.assert_called_once_with(
        'signin',
        config=mock.ANY,
        endpoint_url=mock.ANY,
        verify=False,
    )


@mock.patch('awscli.customizations.login.utils.get_base_sign_in_uri')
@mock.patch(
    'awscli.customizations.login.utils.SameDeviceLoginTokenFetcher.fetch_token'
)
@mock.patch('awscli.customizations.configure.sso.PTKPrompt.get_value')
def test_new_profile_without_region(
    mock_prompt,
    mock_token_fetcher,
    mock_base_sign_in_uri,
    mock_login_command,
    mock_token_loader,
    mock_config_file_writer,
    mock_session,
):
    # Use a different profile than mocked above,
    # simulating a new one via --profile
    mock_session.profile = 'new-profile'
    mock_base_sign_in_uri.return_value = 'https://foo'
    mock_token_fetcher.return_value = (
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
        'arn:aws:iam::0123456789012:user/Admin',
    )

    # Don't set a region via args, rather prompt the user for one
    global_args = Namespace(region=None, endpoint_url=None, verify_ssl=None)
    mock_prompt.return_value = 'us-west-2'

    mock_login_command._run_main(DEFAULT_ARGS, global_args)

    mock_prompt.assert_called_once()
    mock_token_fetcher.assert_called_once()

    mock_config_file_writer.update_config.assert_called_once_with(
        {
            '__section__': "profile new-profile",
            'login_session': 'arn:aws:iam::0123456789012:user/Admin',
            'region': 'us-west-2',
        },
        'configfile',
    )


@pytest.mark.parametrize(
    'profile_config,expected_to_abort',
    [
        pytest.param({}, False, id="Empty profile"),
        pytest.param(
            {'login_session': 'arn:aws:iam::0123456789012:user/Admin'},
            False,
            id="Existing login profile",
        ),
        pytest.param(
            {'web_identity_token_file': '/path'},
            True,
            id="Web Identity Token profile",
        ),
        pytest.param({'sso_role_name': 'role'}, True, id="SSO profile"),
        pytest.param(
            {'aws_access_key_id': 'AKIAIOSFODNN7EXAMPLE'},
            True,
            id="IAM access key profile",
        ),
        pytest.param(
            {'role_arn': 'arn:aws:iam::123456789012:role/MyRole'},
            True,
            id="Assume role profile",
        ),
        pytest.param(
            {'credential_process': '/path/to/credential/process'},
            True,
            id="Credential process profile",
        ),
    ],
)
@mock.patch('awscli.customizations.login.utils.get_base_sign_in_uri')
@mock.patch(
    'awscli.customizations.login.utils.SameDeviceLoginTokenFetcher.fetch_token'
)
def test_abort_if_profile_has_existing_credentials(
    mock_token_fetcher,
    mock_base_sign_in_uri,
    mock_login_command,
    mock_session,
    mock_token_loader,
    profile_config,
    expected_to_abort,
):
    mock_base_sign_in_uri.return_value = 'https://foo'
    mock_token_fetcher.return_value = (
        {
            'accessToken': 'access_token',
            'idToken': SAMPLE_ID_TOKEN,
            'expiresIn': 3600,
        },
        'arn:aws:iam::0123456789012:user/Admin',
    )
    mock_session.full_config = {'profiles': {'profile-name': profile_config}}

    if expected_to_abort:
        with pytest.raises(ConfigurationError):
            mock_login_command._run_main(DEFAULT_ARGS, DEFAULT_GLOBAL_ARGS)
            mock_token_fetcher.assert_not_called()
    else:
        mock_login_command._run_main(DEFAULT_ARGS, DEFAULT_GLOBAL_ARGS)
        mock_token_fetcher.assert_called_once()



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import subprocess
import sys

import pytest

# Commands that should never trigger a prompt_toolkit import.
BASIC_COMMANDS = [
    's3 ls',
    'configure list',
    'logs describe-log-groups',
    'ecs describe-services',
]

_CHECK_SCRIPT = """\
import os
import sys

from unittest.mock import patch
from awscli.botocore.awsrequest import AWSResponse
from awscli.clidriver import create_clidriver

env = {{
    'AWS_DATA_PATH': os.environ.get('AWS_DATA_PATH', ''),
    'AWS_DEFAULT_REGION': 'us-east-1',
    'AWS_ACCESS_KEY_ID': 'testing',
    'AWS_SECRET_ACCESS_KEY': 'testing',
    'AWS_CONFIG_FILE': '',
    'AWS_SHARED_CREDENTIALS_FILE': '',
}}
if os.environ.get('ComSpec'):
    env['ComSpec'] = os.environ['ComSpec']

http_response = AWSResponse(None, 200, {{}}, None)

with patch('os.environ', env), \\
     patch('awscli.botocore.endpoint.Endpoint.make_request',
           return_value=(http_response, {{}})):
    driver = create_clidriver()
    try:
        driver.main({args})
    except SystemExit:
        pass

mods = [m for m in sys.modules if m.startswith('prompt_toolkit')]
if mods:
    print('FAIL: prompt_toolkit modules loaded: ' + ', '.join(sorted(mods)))
    sys.exit(1)
else:
    print('OK')
"""


@pytest.mark.parametrize('cmd', BASIC_COMMANDS)
def test_prompt_toolkit_not_imported(cmd):
    # Historically prompt_toolkit has contributed to significant
    # unnecessary initialization time. This test verifies it
    # is not imported for a handful of commands for which we know
    # it is not needed.
    script = _CHECK_SCRIPT.format(args=repr(cmd.split()))
    # Since prompt_toolkit might be imported during test-running, we execute
    # the commands in a subprocess to ensure we are testing only the modules
    # loaded for the commands under test.
    # We apply a timeout to prevent the subprocess for silently stalling our
    # scripts.
    result = subprocess.run(
        [sys.executable, '-c', script],
        capture_output=True,
        text=True,
        timeout=60,
    )
    assert result.returncode == 0, (
        f"prompt_toolkit was unexpectedly imported for 'aws {cmd}':\n"
        f"stdout: {result.stdout}\nstderr: {result.stderr}"
    )



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import base64
import datetime

import awscli.customizations.ec2.bundleinstance
from awscli.compat import StringIO
from awscli.testutils import BaseAWSCommandParamsTest, mock


class TestBundleInstance(BaseAWSCommandParamsTest):
    prefix = 'ec2 bundle-instance'

    POLICY = (
        b'{"expiration": "2013-08-10T00:00:00.000000Z",'
        b'"conditions": [{"bucket": "mybucket"},{"acl": '
        b'"ec2-bundle-read"},["starts-with", "$key", "foobar"]]}'
    )
    POLICY_SIGNATURE = 'ynxybUMv9YuGbPl7HZ8AFJW/2t0='

    def setUp(self):
        super(TestBundleInstance, self).setUp()
        # This mocks out datetime.datetime.utcnow() so that it always
        # returns the same datetime object.  This is because this value
        # is embedded into the policy file that is generated and we
        # don't what the policy or its signature to change each time
        # we run the test.
        self.datetime_patcher = mock.patch.object(
            awscli.customizations.ec2.bundleinstance.datetime,
            'datetime',
            mock.Mock(wraps=datetime.datetime),
        )
        mocked_datetime = self.datetime_patcher.start()
        mocked_datetime.utcnow.return_value = datetime.datetime(2013, 8, 9)

    def tearDown(self):
        super(TestBundleInstance, self).tearDown()
        self.datetime_patcher.stop()

    def test_no_policy_provided(self):
        args = ' --instance-id i-12345678 --owner-akid AKIAIOSFODNN7EXAMPLE'
        args += ' --owner-sak wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'
        args += ' --bucket mybucket --prefix foobar'
        args_list = (self.prefix + args).split()
        result = {
            'InstanceId': 'i-12345678',
            'Storage': {
                'S3': {
                    'Bucket': 'mybucket',
                    'Prefix': 'foobar',
                    'AWSAccessKeyId': 'AKIAIOSFODNN7EXAMPLE',
                    'UploadPolicy': self.POLICY,
                    'UploadPolicySignature': self.POLICY_SIGNATURE,
                }
            },
        }
        self.assert_params_for_cmd(args_list, result)

    def test_policy_provided(self):
        policy = '{"notarealpolicy":true}'
        base64policy = (
            base64.encodebytes(policy.encode('latin-1'))
            .strip()
            .decode('utf-8')
        )
        policy_signature = 'a5SmoLOxoM0MHpOdC25nE7KIafg='
        args = ' --instance-id i-12345678 --owner-akid AKIAIOSFODNN7EXAMPLE'
        args += ' --owner-sak wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'
        args += ' --bucket mybucket --prefix foobar --policy %s' % base64policy
        args_list = (self.prefix + args).split()
        result = {
            'InstanceId': 'i-12345678',
            'Storage': {
                'S3': {
                    'Bucket': 'mybucket',
                    'Prefix': 'foobar',
                    'AWSAccessKeyId': 'AKIAIOSFODNN7EXAMPLE',
                    'UploadPolicy': b'{"notarealpolicy":true}',
                    'UploadPolicySignature': policy_signature,
                }
            },
        }
        self.assert_params_for_cmd(args_list, result)

    def test_both(self):
        captured = StringIO()
        json = """{"S3":{"Bucket":"foobar","Prefix":"fiebaz"}}"""
        args = (
            ' --instance-id i-12345678 --owner-akid blah --owner-sak blah --storage %s'
            % json
        )
        args_list = (self.prefix + args).split()
        _, stderr, _ = self.assert_params_for_cmd(args_list, expected_rc=252)
        self.assertIn('Mixing the --storage option', stderr)



# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscrt.s3 import S3RequestType

from tests.functional.s3 import (
    BaseCRTTransferClientTest,
    BaseS3TransferCommandTest,
)


class TestRmCommand(BaseS3TransferCommandTest):
    prefix = 's3 rm'

    def test_operations_used(self):
        cmdline = '%s s3://bucket/key.txt' % self.prefix
        self.run_cmd(cmdline, expected_rc=0)
        # The only operation we should have called is DeleteObject.
        self.assertEqual(
            len(self.operations_called), 1, self.operations_called
        )
        self.assertEqual(self.operations_called[0][0].name, 'DeleteObject')

    def test_dryrun_delete(self):
        self.parsed_responses = [self.head_object_response()]
        cmdline = f'{self.prefix} s3://bucket/key.txt --dryrun'
        stdout, _, _ = self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called([])
        self.assertIn('(dryrun) delete: s3://bucket/key.txt', stdout)

    def test_delete_with_request_payer(self):
        cmdline = '%s s3://mybucket/mykey --request-payer' % self.prefix
        self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called(
            [
                (
                    'DeleteObject',
                    {
                        'Bucket': 'mybucket',
                        'Key': 'mykey',
                        'RequestPayer': 'requester',
                    },
                )
            ]
        )

    def test_recursive_delete_with_requests(self):
        cmdline = '%s s3://mybucket/ --recursive --request-payer' % self.prefix
        self.parsed_responses = [
            self.list_objects_response(['mykey']),
            self.empty_response(),
        ]
        self.run_cmd(cmdline, expected_rc=0)
        self.assert_operations_called(
            [
                self.list_objects_request(
                    'mybucket', RequestPayer='requester'
                ),
                self.delete_object_request(
                    'mybucket', 'mykey', RequestPayer='requester'
                ),
            ]
        )


class TestRmWithCRTClient(BaseCRTTransferClientTest):
    def test_delete_using_crt_client(self):
        cmdline = ['s3', 'rm', 's3://bucket/key']
        self.run_command(cmdline)
        crt_requests = self.get_crt_make_request_calls()
        self.assertEqual(len(crt_requests), 1)
        self.assert_crt_make_request_call(
            crt_requests[0],
            expected_type=S3RequestType.DEFAULT,
            expected_host=self.get_virtual_s3_host('bucket'),
            expected_path='/key',
            expected_http_method='DELETE',
        )

    def test_recursive_delete_using_crt_client(self):
        cmdline = ['s3', 'rm', 's3://bucket/', '--recursive']
        self.add_botocore_list_objects_response(['key1', 'key2'])
        self.run_command(cmdline)
        crt_requests = self.get_crt_make_request_calls()
        self.assertEqual(len(crt_requests), 2)
        self.assert_crt_make_request_call(
            crt_requests[0],
            expected_type=S3RequestType.DEFAULT,
            expected_host=self.get_virtual_s3_host('bucket'),
            expected_path='/key1',
            expected_http_method='DELETE',
        )
        self.assert_crt_make_request_call(
            crt_requests[1],
            expected_type=S3RequestType.DEFAULT,
            expected_host=self.get_virtual_s3_host('bucket'),
            expected_path='/key2',
            expected_http_method='DELETE',
        )



import asyncio
import base64
import collections
import contextlib
import copy
import dataclasses
import os
import sys
import threading
import time
from io import BytesIO

# Both nose and py.test will add the first parent directory it
# encounters that does not have a __init__.py to the sys.path. In
# our case, this is the root of the repository. This means that Python
# will import the awscli package from source instead of any installed
# distribution. This environment variable provides the option to remove the
# repository root from sys.path to be able to rely on the installed
# distribution when running the tests.
if os.environ.get('TESTS_REMOVE_REPO_ROOT_FROM_PATH'):
    rootdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path = [
        path
        for path in sys.path
        if not os.path.isdir(path) or not os.path.samefile(path, rootdir)
    ]

import awscli  # noqa: I001
from awscli.clidriver import create_clidriver, AWSCLIEntryPoint
from awscli.compat import collections_abc
from awscli.testutils import (
    unittest,
    mock,
    capture_output,
    if_windows,
    skip_if_windows,
    create_bucket,
    FileCreator,
    ConsistencyWaiter,
    create_dir_bucket,
)

import botocore.awsrequest
import botocore.loaders
import botocore.model
import botocore.serialize
import botocore.validate
from botocore.exceptions import ClientError, WaiterError

from awscrt.crypto import RSA

import prompt_toolkit
import prompt_toolkit.buffer
import prompt_toolkit.input
import prompt_toolkit.output
import prompt_toolkit.input.defaults
import prompt_toolkit.keys
import prompt_toolkit.utils
import prompt_toolkit.key_binding.key_processor
from prompt_toolkit.input.ansi_escape_sequences import REVERSE_ANSI_SEQUENCES

# Botocore testing utilities that we want to preserve import statements for
# in botocore specific tests.
from tests.utils.botocore import (
    assert_url_equal,
    create_session,
    random_chars,
    temporary_file,
    patch_load_service_model,
    ALL_SERVICES,
    BaseEnvVar,
    BaseSessionTest,
    BaseClientDriverTest,
    StubbedSession,
    ClientHTTPStubber,
    SessionHTTPStubber,
    IntegerRefresher,
    FreezeTime,
    get_botocore_default_config_mapping,
)

# S3transfer testing utilities that we want to preserve import statements for
# in s3transfer specific tests.
from tests.utils.s3transfer import (
    HAS_CRT,
    requires_crt,
    skip_if_using_serial_implementation,
    random_bucket_name,
    assert_files_equal,
    NonSeekableReader,
    NonSeekableWriter,
    StreamWithError,
    RecordingSubscriber,
    FileSizeProvider,
    ETagProvider,
    FullObjectChecksumProvider,
    RecordingOSUtils,
    RecordingExecutor,
    TransferCoordinatorWithInterrupt,
    BaseTaskTest,
    BaseSubmissionTaskTest,
    BaseGeneralInterfaceTest,
    StubbedClientTest,
)

# A shared loader to use for classes in this module. This allows us to
# load models outside of influence of a session and take advantage of
# caching to speed up tests.
_LOADER = botocore.loaders.Loader()


class CLIRunner:
    """Runs CLI commands in a stubbed environment"""

    def __init__(self, env=None, session_stubber=None):
        if env is None:
            env = self._get_default_env()
        self.env = env
        if session_stubber is None:
            session_stubber = SessionStubber()
        self._session_stubber = session_stubber

    def run(self, cmdline):
        with mock.patch('os.environ', self.env):
            with capture_output() as output:
                runner_result = self._do_run(cmdline)
                runner_result.stdout = output.stdout.getvalue()
                runner_result.stderr = output.stderr.getvalue()
                return runner_result

    def add_response(self, response):
        self._session_stubber.add_response(response)

    def _get_default_env(self):
        # awscli/__init__.py injects AWS_DATA_PATH at import time
        # so that we can find cli.json.  This might be fixed in the
        # future, but for now we are just replicating the logic in
        # this abstraction.
        cli_data_dir = os.path.join(
            os.path.dirname(os.path.abspath(awscli.__file__)), 'data'
        )
        return {
            'AWS_DATA_PATH': cli_data_dir,
            'AWS_DEFAULT_REGION': 'us-west-2',
            'AWS_ACCESS_KEY_ID': 'access_key',
            'AWS_SECRET_ACCESS_KEY': 'secret_key',
            'AWS_CONFIG_FILE': '',
            'AWS_SHARED_CREDENTIALS_FILE': '',
        }

    def _do_run(self, cmdline):
        driver = create_clidriver()
        entry_point = AWSCLIEntryPoint(driver)
        self._session_stubber.register(driver.session)
        rc = entry_point.main(cmdline)
        self._session_stubber.assert_no_remaining_responses()
        runner_result = CLIRunnerResult(rc)
        runner_result.aws_requests = copy.copy(
            self._session_stubber.received_aws_requests
        )
        return runner_result


class SessionStubber:
    def __init__(self):
        self.received_aws_requests = []
        self._responses = collections.deque()

    def register(self, session):
        events = session.get_component('event_emitter')
        events.register_first(
            'before-parameter-build.*.*',
            self._capture_aws_request,
        )
        events.register_last('request-created', self._capture_http_request)
        events.register_first(
            'before-send.*.*',
            self._return_queued_http_response,
        )

    def add_response(self, response):
        self._responses.append(response)

    def assert_no_remaining_responses(self):
        if len(self._responses) != 0:
            raise AssertionError(
                "The following queued responses are remaining: %s"
                % self._responses
            )

    def _capture_aws_request(self, params, model, context, **kwargs):
        aws_request = AWSRequest(
            service_name=model.service_model.service_name,
            operation_name=model.name,
            params=params,
        )
        self.received_aws_requests.append(aws_request)
        context['current_aws_request'] = aws_request

    def _capture_http_request(self, request, **kwargs):
        request.context['current_aws_request'].http_requests.append(
            HTTPRequest(
                method=request.method,
                url=request.url,
                headers=request.headers,
                body=request.body,
            )
        )

    def _return_queued_http_response(self, request, **kwargs):
        response = self._responses.popleft()
        return response.on_http_request_sent(request)


class BaseResponse:
    def on_http_request_sent(self, request):
        raise NotImplementedError('on_http_request_sent')


class AWSResponse(BaseResponse):
    def __init__(
        self, service_name, operation_name, parsed_response, validate=True
    ):
        self._service_name = service_name
        self._operation_name = operation_name
        self._parsed_response = parsed_response
        self._service_model = self._get_service_model()
        self._operation_model = self._service_model.operation_model(
            self._operation_name
        )
        if validate:
            self._validate_parsed_response()

    def on_http_request_sent(self, request):
        return self._generate_http_response()

    def __repr__(self):
        return (
            'AWSResponse(service_name=%r, operation_name=%r, '
            'parsed_response=%r)'
            % (self._service_name, self._operation_name, self._parsed_response)
        )

    def _get_service_model(self):
        loaded_service_model = _LOADER.load_service_model(
            service_name=self._service_name, type_name='service-2'
        )
        return botocore.model.ServiceModel(
            loaded_service_model, service_name=self._service_name
        )

    def _validate_parsed_response(self):
        if self._operation_model.output_shape:
            botocore.validate.validate_parameters(
                self._parsed_response, self._operation_model.output_shape
            )

    def _generate_http_response(self):
        serialized = self._reverse_serialize_parsed_response()
        return HTTPResponse(
            headers=serialized['headers'], body=serialized['body']
        )

    def _reverse_serialize_parsed_response(self):
        # NOTE: This is fairly hacky, but it gets us a reasonable,
        # serialized response with a fairly low amount of effort. Basically,
        # we swap the operation model so that its input shape points to its
        # output shape so that we can use the serializer to reverse the
        # parsing logic and generate a raw HTTP response instead of a raw HTTP
        # request.
        #
        # Theoretically this should work for many use cases (e.g. JSON
        # protocols), but there are definitely edge cases that are not
        # being handled (e.g. query protocol). Going forward as more tests
        # adopt this, we **will** have to build up the logic around this.
        serializer = botocore.serialize.create_serializer(
            protocol_name=self._service_model.metadata['protocol'],
            include_validation=False,
        )
        self._operation_model.input_shape = self._operation_model.output_shape
        return serializer.serialize_to_request(
            self._parsed_response, self._operation_model
        )


class HTTPResponse(BaseResponse):
    def __init__(self, status_code=200, headers=None, body=b''):
        self.status_code = status_code
        if headers is None:
            headers = {}
        self.headers = headers
        self.body = body
        # Botocore's interface uses content and raw instead of body so just
        # making the content and raw aliases to the body.
        self.content = body
        self.raw = body

    def on_http_request_sent(self, request):
        return self


class CLIRunnerResult:
    def __init__(self, rc, stdout=None, stderr=None):
        self.rc = rc
        self.stdout = stdout
        self.stderr = stderr
        self.aws_requests = []


class AWSRequest:
    def __init__(self, service_name, operation_name, params):
        self.service_name = service_name
        self.operation_name = operation_name
        self.params = params
        self.http_requests = []

    def __repr__(self):
        return 'AWSRequest(service_name=%r, operation_name=%r, params=%r)' % (
            self.service_name,
            self.operation_name,
            self.params,
        )

    def __eq__(self, other):
        return (
            self.service_name == other.service_name
            and self.operation_name == other.operation_name
            and self.params == other.params
        )

    def __ne__(self, other):
        return not self.__eq__(other)


class HTTPRequest:
    def __init__(self, method, url, headers, body):
        self.method = method
        self.url = url
        self.headers = headers
        self.body = body

    def __repr__(self):
        return 'HTTPRequest(method=%r, url=%r, headers=%r, body=%r)' % (
            self.method,
            self.url,
            self.headers,
            self.body,
        )

    def __eq__(self, other):
        return (
            self.method == other.method
            and self.url == other.url
            and self.headers == other.headers
            and self.body == other.body
        )

    def __ne__(self, other):
        return not self.__eq__(other)


# CaseInsensitiveDict from requests that must be serializble.
class CaseInsensitiveDict(collections_abc.MutableMapping):
    def __init__(self, data=None, **kwargs):
        self._store = dict()
        if data is None:
            data = {}
        self.update(data, **kwargs)

    def __setitem__(self, key, value):
        # Use the lowercased key for lookups, but store the actual
        # key alongside the value.
        self._store[key.lower()] = (key, value)

    def __getitem__(self, key):
        return self._store[key.lower()][1]

    def __delitem__(self, key):
        del self._store[key.lower()]

    def __iter__(self):
        return (casedkey for casedkey, mappedvalue in self._store.values())

    def __len__(self):
        return len(self._store)

    def lower_items(self):
        """Like iteritems(), but with all lowercase keys."""
        return (
            (lowerkey, keyval[1]) for (lowerkey, keyval) in self._store.items()
        )

    def __eq__(self, other):
        if isinstance(other, collections_abc.Mapping):
            other = CaseInsensitiveDict(other)
        else:
            return NotImplemented
        # Compare insensitively
        return dict(self.lower_items()) == dict(other.lower_items())

    # Copy is required
    def copy(self):
        return CaseInsensitiveDict(self._store.values())

    def __repr__(self):
        return str(dict(self.items()))


class RawResponse(BytesIO):
    # TODO: There's a few objects similar to this in various tests, let's
    # try and consolidate to this one in a future commit.
    def stream(self, **kwargs):
        contents = self.read()
        while contents:
            yield contents
            contents = self.read()


@dataclasses.dataclass
class AppRunContext:
    return_value = None
    raised_exception = None


class PromptToolkitAppRunner:
    _EVENT_WAIT_TIMEOUT = 5

    def __init__(self, app, pre_run=None):
        self.app = app
        self._pre_run = pre_run
        self._done_rendering_event = threading.Event()
        self.app.after_render = prompt_toolkit.utils.Event(
            self.app, self._notify_done_rendering
        )
        self._done_completing_event = threading.Event()

    @contextlib.contextmanager
    def run_app_in_thread(self, target=None, args=None):
        if target is None:
            target = self.app.run
        if args is None:
            args = (self._pre_run,)

        run_context = AppRunContext()
        thread = threading.Thread(
            target=self._do_run_app, args=(target, args, run_context)
        )
        try:
            thread.start()
            self._wait_until_app_is_done_updating()
            yield run_context
        finally:
            if self._app_is_exitable():
                self.app.exit()
            thread.join()

    def feed_input(self, *keys):
        for key in keys:
            self._done_rendering_event.clear()
            self.app.input.send_text(self._convert_key_to_vt100_data(key))
            self._wait_until_app_is_done_updating()

    def wait_for_completions_on_current_buffer(self):
        if self._current_buffer_has_completions():
            return
        self.app.current_buffer.on_completions_changed.add_handler(
            self._notify_done_completing
        )
        self._done_completing_event.wait(self._EVENT_WAIT_TIMEOUT)
        self._done_completing_event.clear()

    def _do_run_app(self, target, target_args, app_run_context):
        # This is the function that will be passed to our thread to
        # actually run the application
        try:
            # When we run the app in a separate thread, there is no
            # default event loop. This ensures we create one as it is
            # likely the application will try to grab the default loop
            loop = asyncio.new_event_loop()
            asyncio.set_event_loop(loop)
            app_run_context.return_value = target(*target_args)
        except BaseException as e:
            app_run_context.raised_exception = e
        finally:
            loop.close()

    def _wait_until_app_is_done_updating(self):
        self._wait_until_app_is_done_rendering()
        # Generally it is not a safe assumption to make that once the
        # app is done rendering the UI will be in its final state.
        # It is possible that because of the rendering it triggers another
        # redraw of the application. So here we manually invalidate the app
        # to flush out any pending changes to the UI and then wait for those
        # changes to be rendered.
        self.app.invalidate()
        self._wait_until_app_is_done_rendering()

    def _wait_until_app_is_done_rendering(self):
        self._done_rendering_event.wait(self._EVENT_WAIT_TIMEOUT)
        self._done_rendering_event.clear()

    def _notify_done_rendering(self, app):
        self._done_rendering_event.set()

    def _notify_done_completing(self, app):
        self._done_completing_event.set()

    def _current_buffer_has_completions(self):
        return (
            self.app.current_buffer.complete_state
            and self.app.current_buffer.complete_state.completions
        )

    def _app_is_exitable(self):
        # This needs to be used instead of app.done() because prompt toolkit
        # sets the future to None when it finishes run_async. So it is
        # indeterminable whether app.done() is actually done because
        # the future being None results in a return value of False.
        # So instead we have our own custom check to see if there is a future
        # present on the app (meaning the run_async has not finished) and the
        # result for that future has not been set.
        return self.app.future and not self.app.future.done()

    def _convert_key_to_vt100_data(self, key):
        return REVERSE_ANSI_SEQUENCES.get(key, key)


class S3Utils:
    _PUT_HEAD_SHARED_EXTRAS = [
        'SSECustomerAlgorithm',
        'SSECustomerKey',
        'SSECustomerKeyMD5',
        'RequestPayer',
    ]

    def __init__(self, session, region=None):
        self._session = session
        self._region = region
        self._bucket_to_region = {}
        self._client = self._session.create_client(
            's3', region_name=self._region
        )

    def _create_client_for_bucket(self, bucket_name):
        region = self._bucket_to_region.get(bucket_name, self._region)
        client = self._session.create_client('s3', region_name=region)
        return client

    def assert_key_contents_equal(self, bucket, key, expected_contents):
        self.wait_until_key_exists(bucket, key)
        if isinstance(expected_contents, BytesIO):
            expected_contents = expected_contents.getvalue().decode('utf-8')
        actual_contents = self.get_key_contents(bucket, key)
        # The contents can be huge so we try to give helpful error messages
        # without necessarily printing the actual contents.
        assert len(actual_contents) == len(expected_contents)
        assert actual_contents == expected_contents, (
            f"Contents for {bucket}/{key} do not match (but they "
            f"have the same length)"
        )

    def create_bucket(self, name=None, region=None):
        if not region:
            region = self._region
        bucket_name = create_bucket(self._session, name, region)
        self._bucket_to_region[bucket_name] = region
        # Wait for the bucket to exist before letting it be used.
        self.wait_bucket_exists(bucket_name)
        return bucket_name

    def create_dir_bucket(self, name=None, location=None):
        if location:
            region, _ = location
        else:
            region = self._region
        bucket_name = create_dir_bucket(self._session, name, location)
        self._bucket_to_region[bucket_name] = region
        # Wait for the bucket to exist before letting it be used.
        self.wait_bucket_exists(bucket_name)
        return bucket_name

    def put_object(self, bucket_name, key_name, contents='', extra_args=None):
        client = self._create_client_for_bucket(bucket_name)
        call_args = {'Bucket': bucket_name, 'Key': key_name, 'Body': contents}
        if extra_args is not None:
            call_args.update(extra_args)
        response = client.put_object(**call_args)
        extra_head_params = {}
        if extra_args:
            extra_head_params = dict(
                (k, v)
                for (k, v) in extra_args.items()
                if k in self._PUT_HEAD_SHARED_EXTRAS
            )
        self.wait_until_key_exists(
            bucket_name,
            key_name,
            extra_params=extra_head_params,
        )
        return response

    def delete_bucket(self, bucket_name, attempts=5, delay=5):
        self.remove_all_objects(bucket_name)
        client = self._create_client_for_bucket(bucket_name)

        # There's a chance that, even though the bucket has been used
        # several times, the delete will fail due to eventual consistency
        # issues.
        attempts_remaining = attempts
        while True:
            attempts_remaining -= 1
            try:
                client.delete_bucket(Bucket=bucket_name)
                break
            except client.exceptions.NoSuchBucket:
                if self.bucket_not_exists(bucket_name):
                    # Fast fail when the NoSuchBucket error is real.
                    break
                if attempts_remaining <= 0:
                    raise
                time.sleep(delay)

        self._bucket_to_region.pop(bucket_name, None)

    def remove_all_objects(self, bucket_name):
        client = self._create_client_for_bucket(bucket_name)
        paginator = client.get_paginator('list_objects_v2')
        pages = paginator.paginate(Bucket=bucket_name)
        key_names = []
        for page in pages:
            key_names += [obj['Key'] for obj in page.get('Contents', [])]
        for key_name in key_names:
            self.delete_key(bucket_name, key_name)

    def delete_key(self, bucket_name, key_name):
        client = self._create_client_for_bucket(bucket_name)
        client.delete_object(Bucket=bucket_name, Key=key_name)

    def get_key_contents(self, bucket_name, key_name):
        self.wait_until_key_exists(bucket_name, key_name)
        client = self._create_client_for_bucket(bucket_name)
        response = client.get_object(Bucket=bucket_name, Key=key_name)
        return response['Body'].read().decode('utf-8')

    def wait_bucket_exists(self, bucket_name, min_successes=3):
        client = self._create_client_for_bucket(bucket_name)
        waiter = client.get_waiter('bucket_exists')
        consistency_waiter = ConsistencyWaiter(
            min_successes=min_successes, delay_initial_poll=True
        )
        consistency_waiter.wait(
            lambda: waiter.wait(Bucket=bucket_name) is None
        )

    def bucket_not_exists(self, bucket_name):
        client = self._create_client_for_bucket(bucket_name)
        try:
            client.head_bucket(Bucket=bucket_name)
            return True
        except ClientError as error:
            if error.response.get('Code') == '404':
                return False
            raise

    def key_exists(self, bucket_name, key_name, min_successes=3):
        try:
            self.wait_until_key_exists(
                bucket_name, key_name, min_successes=min_successes
            )
            return True
        except (ClientError, WaiterError):
            return False

    def key_not_exists(self, bucket_name, key_name, min_successes=3):
        try:
            self.wait_until_key_not_exists(
                bucket_name, key_name, min_successes=min_successes
            )
            return True
        except (ClientError, WaiterError):
            return False

    def list_multipart_uploads(self, bucket_name):
        client = self._create_client_for_bucket(bucket_name)
        return client.list_multipart_uploads(Bucket=bucket_name).get(
            'Uploads', []
        )

    def list_buckets(self):
        response = self._client.list_buckets()
        return response['Buckets']

    def get_bucket_website(self, bucket_name):
        client = self._create_client_for_bucket(bucket_name)
        return client.get_bucket_website(Bucket=bucket_name)

    def content_type_for_key(self, bucket_name, key_name):
        parsed = self.head_object(bucket_name, key_name)
        return parsed['ContentType']

    def head_object(self, bucket_name, key_name):
        client = self._create_client_for_bucket(bucket_name)
        response = client.head_object(Bucket=bucket_name, Key=key_name)
        return response

    def wait_until_key_exists(
        self, bucket_name, key_name, extra_params=None, min_successes=3
    ):
        self._wait_for_key(
            bucket_name, key_name, extra_params, min_successes, exists=True
        )

    def wait_until_key_not_exists(
        self, bucket_name, key_name, extra_params=None, min_successes=3
    ):
        self._wait_for_key(
            bucket_name, key_name, extra_params, min_successes, exists=False
        )

    def _wait_for_key(
        self,
        bucket_name,
        key_name,
        extra_params=None,
        min_successes=3,
        exists=True,
    ):
        client = self._create_client_for_bucket(bucket_name)
        if exists:
            waiter = client.get_waiter('object_exists')
        else:
            waiter = client.get_waiter('object_not_exists')
        params = {'Bucket': bucket_name, 'Key': key_name}
        if extra_params is not None:
            params.update(extra_params)
        for _ in range(min_successes):
            waiter.wait(**params)


class PublicPrivateKeyLoader:
    def load_private_key_and_public_key(private_key_path, public_key_path):
        with open(private_key_path, 'rb') as f:
            private_key_byte_input = f.read()

        private_key = RSA.new_private_key_from_pem_data(private_key_byte_input)

        with open(public_key_path, 'rb') as f:
            public_key_byte_input = f.read()

        public_key_b64 = base64.b64encode(public_key_byte_input)

        return public_key_b64, private_key



# Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestCreateCustomVerificationEmailTemplate(BaseAWSCommandParamsTest):
    prefix = 'ses create-custom-verification-email-template'
    template_name = 'test-template-name'
    from_email_address = 'from@example.com'
    template_subject = 'template-subject'
    template_content = 'template-content'
    success_redirection_url = 'https://aws.amazon.com/ses/verifysuccess'
    failure_redirection_url = 'https://aws.amazon.com/ses/verifyfailure'

    def test_create_custom_verification_email_template(self):
        cmdline = self.prefix
        cmdline += ' --template-name ' + self.template_name
        cmdline += ' --from-email-address ' + self.from_email_address
        cmdline += ' --template-subject ' + self.template_subject
        cmdline += ' --template-content ' + self.template_content
        cmdline += ' --success-redirection-url ' + self.success_redirection_url
        cmdline += ' --failure-redirection-url ' + self.failure_redirection_url

        result = {
            'TemplateName': self.template_name,
            'FromEmailAddress': self.from_email_address,
            'TemplateSubject': self.template_subject,
            'TemplateContent': self.template_content,
            'SuccessRedirectionURL': self.success_redirection_url,
            'FailureRedirectionURL': self.failure_redirection_url,
        }
        self.assert_params_for_cmd(cmdline, result)



# Copyright 2012-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from argparse import Namespace

from botocore.compat import json

from awscli.customizations.servicecatalog import exceptions
from awscli.customizations.servicecatalog.generateprovisioningartifact import (
    GenerateProvisioningArtifactCommand,
)
from awscli.testutils import capture_output, mock, unittest


class TestCreateProvisioningArtifactCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.servicecatalog_client = mock.Mock()
        self.s3_client = mock.Mock()
        self.session.create_client.side_effect = [
            self.s3_client,
            self.servicecatalog_client,
        ]
        self.session.get_available_regions.return_value = [
            'us-east-1',
            'eu-west-1',
        ]
        self.cmd = GenerateProvisioningArtifactCommand(self.session)

        self.args = Namespace()
        self.args.file_path = 'foo-file-path'
        self.args.bucket_name = 'foo-bucket-name'
        self.args.provisioning_artifact_name = 'foo-pa-name'
        self.args.provisioning_artifact_description = 'foo-pa-desc'
        self.args.provisioning_artifact_type = 'CLOUD_FORMATION_TEMPLATE'
        self.args.product_id = 'prod-1234567890abc'

        self.s3_url = "https://s3.amazonaws.com/foo-bucket-name/foo-file-path"

        # set global args
        self.global_args = Namespace()
        self.global_args.region = 'us-east-1'
        self.global_args.endpoint_url = None
        self.global_args.verify_ssl = None

    @mock.patch('os.path.getsize', return_value=1)
    def test_happy_path(self, getsize_patch):
        # Arrange
        self.servicecatalog_client.create_provisioning_artifact.return_value = self.get_create_provisioning_artifact_output()
        expected_pa_detail = self.get_create_provisioning_artifact_output()
        del expected_pa_detail['ResponseMetadata']
        expected_response_output = json.dumps(
            expected_pa_detail, indent=2, ensure_ascii=False
        )

        # Act
        with capture_output() as captured:
            result = self.cmd._run_main(self.args, self.global_args)

        # Assert
        self.session.create_client.assert_called_with(
            'servicecatalog',
            region_name=self.global_args.region,
            endpoint_url=None,
            verify=None,
        )
        self.servicecatalog_client.create_provisioning_artifact.assert_called_once_with(
            ProductId=self.args.product_id,
            Parameters=self.get_provisioning_artifact_parameters(
                self.args.provisioning_artifact_name,
                self.args.provisioning_artifact_description,
                self.args.provisioning_artifact_type,
            ),
        )
        self.assertEqual(expected_response_output, captured.stdout.getvalue())
        self.assertEqual(0, result)

    @mock.patch('os.path.getsize', return_value=1)
    def test_happy_path_unicode(self, getsize_patch):
        # Arrange
        self.args.provisioning_artifact_name = '\u05d1\u05e8\u05d9\u05e6'
        self.args.provisioning_artifact_description = '\u00fd\u00a9\u0194'
        self.servicecatalog_client.create_provisioning_artifact.return_value = self.get_create_provisioning_artifact_output()
        expected_pa_detail = self.get_create_provisioning_artifact_output()
        del expected_pa_detail['ResponseMetadata']
        expected_response_output = json.dumps(
            expected_pa_detail, indent=2, ensure_ascii=False
        )

        # Act
        with capture_output() as captured:
            result = self.cmd._run_main(self.args, self.global_args)

        # Assert
        self.session.create_client.assert_called_with(
            'servicecatalog',
            region_name=self.global_args.region,
            endpoint_url=None,
            verify=None,
        )
        self.servicecatalog_client.create_provisioning_artifact.assert_called_once_with(
            ProductId=self.args.product_id,
            Parameters=self.get_provisioning_artifact_parameters(
                self.args.provisioning_artifact_name,
                self.args.provisioning_artifact_description,
                self.args.provisioning_artifact_type,
            ),
        )
        self.assertEqual(expected_response_output, captured.stdout.getvalue())
        self.assertEqual(0, result)

    def test_region_not_supported(self):
        self.global_args.region = 'not-supported-region'
        with self.assertRaisesRegex(
            exceptions.InvalidParametersException, "not supported"
        ):
            self.cmd._run_main(self.args, self.global_args)

    def get_provisioning_artifact_parameters(
        self, pa_name, pa_description, pa_type
    ):
        return {
            'Name': pa_name,
            'Description': pa_description,
            'Info': {'LoadTemplateFromURL': self.s3_url},
            'Type': pa_type,
        }

    def get_create_provisioning_artifact_output(self):
        return {
            'Info': {'TemplateUrl': self.s3_url},
            'ProvisioningArtifactDetail': {
                'Description': self.args.provisioning_artifact_description,
                'Id': 'pa-inifmhpr47ft2',
                'Name': self.args.provisioning_artifact_name,
                'Type': self.args.provisioning_artifact_type,
            },
            'Status': "CREATING",
            'ResponseMetadata': {},
        }



# Copyright 2012-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from awscli.customizations.servicecatalog import (
    GenerateCommand,
    inject_commands,
    register_servicecatalog_commands,
)
from awscli.testutils import mock, unittest


class TestRegisterServiceCatalogCommands(unittest.TestCase):
    def test_register_servicecatalog_commands(self):
        event_emitter = mock.Mock()
        register_servicecatalog_commands(event_emitter)
        event_emitter.register.assert_has_calls(
            [
                mock.call(
                    'building-command-table.servicecatalog', inject_commands
                )
            ]
        )


class TestInjectCommands(unittest.TestCase):
    def test_inject_commands(self):
        command_table = {}
        session = mock.Mock()
        inject_commands(command_table, session)
        self.assertIn('generate', command_table)
        self.assertIsInstance(command_table['generate'], GenerateCommand)



# Copyright 2012-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from argparse import Namespace

from awscli.customizations.exceptions import ParamValidationError
from awscli.customizations.servicecatalog import GenerateCommand
from awscli.testutils import unittest


class TestGenerateCommand(unittest.TestCase):
    def setUp(self):
        self.cmd = GenerateCommand(None)

    def test_no_subcommand(self):
        # Arrange
        arguments = Namespace()
        arguments.subcommand = None

        # Act+Assert
        with self.assertRaises(ParamValidationError):
            self.cmd._run_main(arguments, None)

    def test_with_subcommand(self):
        # Arrange
        arguments = Namespace(subcommand="product")

        # Act
        self.cmd._run_main(arguments, None)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import contextlib
import json
import logging
import os

import pytest
from ruamel.yaml import YAML

from awscli.clidriver import create_clidriver
from awscli.testutils import mock


# NOTE: This should be a standalone pytest fixture.  However, fixtures cannot
# be used outside of other fixtures or test cases, and it is needed by
# get_all_cli_skeleton_commands to generate the parameterization cases.
# So the environ patching logic is extracted out to this general helper
# context manager that can be used in both case generator and an actual
# fixture for the tests.
@contextlib.contextmanager
def patch_environ():
    environ = {
        'AWS_DATA_PATH': os.environ['AWS_DATA_PATH'],
        'AWS_DEFAULT_REGION': 'us-east-1',
        'AWS_ACCESS_KEY_ID': 'access_key',
        'AWS_SECRET_ACCESS_KEY': 'secret_key',
        'AWS_CONFIG_FILE': '',
        'AWS_SHARED_CREDENTIALS_FILE': '',
    }
    with mock.patch('os.environ', environ):
        yield


@pytest.fixture
def clean_environ():
    with patch_environ():
        yield


@pytest.fixture
def yaml_safe_loader():
    return YAML(typ="safe", pure=True)


def get_all_cli_skeleton_commands():
    skeleton_commands = []
    with patch_environ():
        driver = create_clidriver()
        help_command = driver.create_help_command()
        for command_name, command_obj in help_command.command_table.items():
            sub_help = command_obj.create_help_command()
            # This avoids command objects like ``PreviewModeCommand`` that
            # do not exhibit any visible functionality (i.e. provides a command
            # for the CLI).
            if hasattr(sub_help, 'command_table'):
                for sub_name, sub_command in sub_help.command_table.items():
                    op_help = sub_command.create_help_command()
                    arg_table = op_help.arg_table
                    if 'generate-cli-skeleton' in arg_table:
                        skeleton_commands.append(f'{command_name} {sub_name}')
    return skeleton_commands


SKELETON_COMMANDS = get_all_cli_skeleton_commands()


@pytest.mark.parametrize('cmd', SKELETON_COMMANDS)
def test_gen_input_skeleton(cmd, capsys, clean_environ):
    stdout, stderr, _ = _run_cmd(cmd + ' --generate-cli-skeleton', capsys)
    # Test that a valid JSON blob is emitted to stdout is valid.
    try:
        json.loads(stdout)
    except ValueError:
        raise AssertionError(
            f"Could not generate CLI skeleton for command: {cmd}\n"
            f"stdout:\n{stdout}\n"
            f"stderr:\n{stderr}\n"
        )


@pytest.mark.parametrize('cmd', SKELETON_COMMANDS)
def test_gen_yaml_input_skeleton(cmd, capsys, clean_environ, yaml_safe_loader):
    stdout, stderr, _ = _run_cmd(
        cmd + ' --generate-cli-skeleton yaml-input', capsys
    )
    try:
        yaml_safe_loader.load(stdout)
    except ValueError:
        raise AssertionError(
            f"Could not generate CLI YAML skeleton for command: {cmd}\n"
            f"stdout:\n{stdout}\n"
            f"stderr:\n{stderr}\n"
        )


def _run_cmd(cmd, capsys, expected_rc=0):
    logging.debug("Calling cmd: %s", cmd)
    # Drivers do not seem to be reusable since the formatters seem to not clear
    # themselves between runs. This is fine in practice since a driver is only
    # called once but for tests it means we need to create a new driver for
    # each test, which is far more heavyweight than it needs to be. Might be
    # worth seeing if we can make drivers reusable to speed these up generated
    # tests.
    driver = create_clidriver()
    try:
        rc = driver.main(cmd.split())
    except SystemExit as e:
        # We need to catch SystemExit so that we
        # can get a proper rc and still present the
        # stdout/stderr to the test runner so we can
        # figure out what went wrong.
        rc = e.code
    stdout, stderr = capsys.readouterr()
    assert rc == expected_rc, (
        f"Unexpected rc (expected: {expected_rc}, actual: {rc}) "
        f"for command: {cmd}\nstdout:\n{stdout}stderr:\n{stderr}"
    )
    return stdout, stderr, rc



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestAssociateAddress(BaseAWSCommandParamsTest):
    prefix = 'ec2 associate-address'

    def test_basic(self):
        cmdline = self.prefix
        cmdline += ' --instance-id i-12345678'
        cmdline += ' --public-ip 192.168.0.0'
        result = {'InstanceId': 'i-12345678', 'PublicIp': '192.168.0.0'}
        self.assert_params_for_cmd(cmdline, result)

    def test_vpc_basic(self):
        cmdline = self.prefix
        cmdline += ' --instance-id i-12345678'
        cmdline += ' --public-ip 192.168.0.0'
        cmdline += ' --allocation-id eipalloc-12345678'
        cmdline += ' --allow-reassociation'
        result = {
            'InstanceId': 'i-12345678',
            'PublicIp': '192.168.0.0',
            'AllowReassociation': True,
            'AllocationId': 'eipalloc-12345678',
        }
        self.assert_params_for_cmd(cmdline, result)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import decimal

from awscli.testutils import BaseAWSCommandParamsTest


class TestPutMetricData(BaseAWSCommandParamsTest):
    maxDiff = None

    prefix = 'cloudwatch put-metric-data '

    expected_output = {
        'MetricData': [
            {
                'MetricName': 'FreeMemoryBytes',
                'Unit': 'Bytes',
                'Timestamp': '2013-08-22T10:58:12.283Z',
                'Value': decimal.Decimal("9130160128"),
            }
        ],
        'Namespace': '"Foo/Bar"',
    }

    def test_using_json(self):
        args = (
            '--namespace "Foo/Bar" '
            '--metric-data [{"MetricName":"FreeMemoryBytes",'
            '"Unit":"Bytes",'
            '"Timestamp":"2013-08-22T10:58:12.283Z",'
            '"Value":9130160128}]'
        )
        cmdline = self.prefix + args
        self.assert_params_for_cmd(cmdline, self.expected_output)

    def test_using_promoted_params(self):
        # This is equivalent to the json version in test_using_json
        # above.
        args = (
            '--namespace "Foo/Bar" '
            '--metric-name FreeMemoryBytes '
            '--unit Bytes '
            '--timestamp 2013-08-22T10:58:12.283Z '
            '--value 9130160128'
        )
        cmdline = self.prefix + args
        self.assert_params_for_cmd(cmdline, self.expected_output)

    def test_using_shorthand_syntax(self):
        args = (
            '--metric-name PageViewCount '
            '--namespace MyService '
            '--statistic-value Sum=11,Minimum=2,Maximum=5,SampleCount=3 '
            '--timestamp 2014-02-14T12:00:00.000Z'
        )
        cmdline = self.prefix + args
        expected = {
            'MetricData': [
                {
                    'MetricName': 'PageViewCount',
                    'StatisticValues': {
                        'Maximum': decimal.Decimal('5'),
                        'Minimum': decimal.Decimal('2'),
                        'SampleCount': decimal.Decimal('3'),
                        'Sum': decimal.Decimal('11'),
                    },
                    'Timestamp': '2014-02-14T12:00:00.000Z',
                }
            ],
            'Namespace': 'MyService',
        }
        self.assert_params_for_cmd(cmdline, expected)

    def test_using_storage_resolution(self):
        args = (
            '--metric-name Foo '
            '--namespace Bar '
            '--value 5 '
            '--storage-resolution 1 '
        )
        cmdline = self.prefix + args
        expected = {
            'MetricData': [
                {
                    'MetricName': 'Foo',
                    'Value': decimal.Decimal('5'),
                    'StorageResolution': 1,
                }
            ],
            'Namespace': 'Bar',
        }
        self.assert_params_for_cmd(cmdline, expected)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.hooks import HierarchicalEmitter

from awscli.clidriver import CLIDriver
from awscli.customizations.commands import (
    BasicCommand,
    BasicDocHandler,
    BasicHelp,
)
from awscli.testutils import BaseAWSCommandParamsTest, mock, unittest
from tests.unit.test_clidriver import FakeCommand, FakeSession


class MockCustomCommand(BasicCommand):
    NAME = 'mock'

    SUBCOMMANDS = [{'name': 'basic', 'command_class': BasicCommand}]


class TestCommandLoader(unittest.TestCase):
    def test_basic_help_with_contents(self):
        cmd_object = mock.Mock()
        mock_module = mock.Mock()
        mock_module.__file__ = '/some/root'
        cmd_object.DESCRIPTION = BasicCommand.FROM_FILE(
            'foo', 'bar', 'baz.txt', root_module=mock_module
        )
        help_command = BasicHelp(mock.Mock(), cmd_object, {}, {})
        with mock.patch('awscli.customizations.commands._open') as mock_open:
            mock_open.return_value.__enter__.return_value.read.return_value = (
                'fake description'
            )
            self.assertEqual(help_command.description, 'fake description')


class TestBasicCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.command = BasicCommand(self.session)

    def test_load_arg_table_property(self):
        # Ensure ``_build_arg_table()`` is called if it has not been
        # built via the ``arg_table`` property.  It should be an empty
        # dictionary.
        orig_arg_table = self.command.arg_table
        self.assertEqual(orig_arg_table, {})
        # Ensure the ``arg_table`` is not built again if
        # ``arg_table`` property is called again.
        self.assertIs(orig_arg_table, self.command.arg_table)

    def test_load_subcommand_table_property(self):
        # Ensure ``_build_subcommand_table()`` is called if it has not
        # been built via the ``subcommand_table`` property. It should be
        # an empty dictionary.
        orig_subcommand_table = self.command.subcommand_table
        self.assertEqual(orig_subcommand_table, {})
        # Ensure the ``subcommand_table`` is not built again if
        # ``subcommand_table`` property is called again.
        self.assertIs(orig_subcommand_table, self.command.subcommand_table)

    def test_load_lineage(self):
        self.assertEqual(self.command.lineage, [self.command])
        self.assertEqual(self.command.lineage_names, [self.command.name])

    def test_pass_lineage_to_child_command(self):
        self.command = MockCustomCommand(self.session)
        subcommand = self.command.subcommand_table['basic']
        lineage = subcommand.lineage
        self.assertEqual(len(lineage), 2)
        self.assertEqual(lineage[0], self.command)
        self.assertIsInstance(lineage[1], BasicCommand)
        self.assertEqual(
            subcommand.lineage_names, [self.command.name, subcommand.name]
        )

    def test_event_class(self):
        self.command = MockCustomCommand(self.session)
        help_command = self.command.create_help_command()
        self.assertEqual(help_command.event_class, 'mock')
        subcommand = self.command.subcommand_table['basic']
        sub_help_command = subcommand.create_help_command()
        # Note that the name of this Subcommand was never changed even
        # though it was put into the table as ``basic``. If no name
        # is overriden it uses the name ``commandname``.
        self.assertEqual(sub_help_command.event_class, 'mock.commandname')


class TestBasicCommandHooks(unittest.TestCase):
    # These tests verify the proper hooks are emitted from BasicCommand.

    def setUp(self):
        self.session = FakeSession()
        self.session.get_config_variable = mock.Mock()
        return_values = {'cli_auto_prompt': 'off', 'cli_help_output': None}
        self.session.get_config_variable.side_effect = return_values
        self.emitter = mock.Mock(wraps=HierarchicalEmitter())
        self.session.emitter = self.emitter

    def assert_events_fired_in_order(self, events):
        args = self.emitter.emit.call_args_list
        actual_events = [arg[0][0] for arg in args]
        self.assertEqual(actual_events, events)

    def inject_command(self, command_table, **kwargs):
        command = FakeCommand(self.session)
        command.NAME = 'foo'
        command_table['foo'] = command

    def test_expected_events_are_emitted_in_order(self):
        driver = CLIDriver(session=self.session)
        self.emitter.register('building-command-table.s3', self.inject_command)

        driver.main('s3 foo'.split())

        self.assert_events_fired_in_order(
            [
                'building-command-table.main',
                'building-top-level-params',
                'top-level-args-parsed',
                'session-initialized',
                'building-command-table.s3',
                'building-command-table.s3_foo',
                'building-arg-table.s3_foo',
                'before-building-argument-table-parser.s3.foo',
            ]
        )


class TestBasicDocHandler(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.obj = MockCustomCommand(self.session)
        self.command_table = {}
        self.arg_table = {}

    def create_help_command(self):
        return BasicHelp(
            self.session, self.obj, self.command_table, self.arg_table
        )

    def create_arg_table(self):
        return CLIDriver().create_help_command().arg_table

    def generate_global_option_docs(self, help_command):
        operation_handler = BasicDocHandler(help_command)
        operation_handler.doc_global_option(help_command=help_command)
        return help_command.doc.getvalue().decode('utf-8')

    def generate_global_synopsis_docs(self, help_command):
        operation_handler = BasicDocHandler(help_command)
        operation_handler.doc_synopsis_end(help_command=help_command)
        return help_command.doc.getvalue().decode('utf-8')

    def assert_global_args_documented(self, arg_table, content):
        for arg in arg_table:
            self.assertIn(arg_table.get(arg).cli_name, content)

    def assert_global_args_not_documented(self, arg_table, content):
        for arg in arg_table:
            self.assertNotIn(arg_table.get(arg).cli_name, content)

    def test_includes_global_options_when_command_table_empty(self):
        help_command = self.create_help_command()
        arg_table = self.create_arg_table()
        help_command.arg_table = arg_table
        rendered = self.generate_global_option_docs(help_command)
        self.assert_global_args_documented(arg_table, rendered)

    def test_excludes_global_options_when_command_table_not_empty(self):
        help_command = self.create_help_command()
        arg_table = self.create_arg_table()
        help_command.arg_table = arg_table
        fake_command = FakeCommand(FakeSession())
        fake_command.NAME = 'command'
        help_command.command_table = {'command': fake_command}
        rendered = self.generate_global_option_docs(help_command)
        self.assert_global_args_not_documented(arg_table, rendered)

    def test_includes_global_synopsis_when_command_table_empty(self):
        help_command = self.create_help_command()
        arg_table = self.create_arg_table()
        help_command.arg_table = arg_table
        rendered = self.generate_global_synopsis_docs(help_command)
        self.assert_global_args_documented(arg_table, rendered)

    def test_excludes_global_synopsis_when_command_table_not_empty(self):
        help_command = self.create_help_command()
        arg_table = self.create_arg_table()
        help_command.arg_table = arg_table
        fake_command = FakeCommand(FakeSession())
        fake_command.NAME = 'command'
        help_command.command_table = {'command': fake_command}
        rendered = self.generate_global_synopsis_docs(help_command)
        self.assert_global_args_not_documented(arg_table, rendered)


class TestUserAgentCommandSection(BaseAWSCommandParamsTest):
    def _assert_customization_in_user_agent(self, customization):
        self.assertTrue(
            self.driver.session.user_agent_extra.endswith(customization)
        )

    def test_customization_in_user_agent_s3_cp(self):
        cmd = 's3 cp s3://foo s3://bar'
        self.run_cmd(cmd)
        self._assert_customization_in_user_agent(' md/command#s3.cp')

    def test_customization_in_user_agent_s3_ls(self):
        cmd = 's3 ls'
        self.run_cmd(cmd)
        self._assert_customization_in_user_agent(' md/command#s3.ls')

    def test_customization_in_user_agent_logs_tail(self):
        cmd = 'logs tail foo'
        # it should fail but the user_agent should be correct
        self.run_cmd(cmd, expected_rc=255)
        self._assert_customization_in_user_agent(' md/command#logs.tail')

    def test_service_operation_in_user_agent(self):
        cmd = 'ec2 describe-instances'
        self.run_cmd(cmd)
        self._assert_customization_in_user_agent(
            ' md/command#ec2.describe-instances'
        )

    def test_custom_service_operation_in_user_agent(self):
        cmd = 'rds add-option-to-option-group --option-group-name foo'
        self.run_cmd(cmd)
        self._assert_customization_in_user_agent(
            ' md/command#rds.add-option-to-option-group'
        )



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import errno
import json
import subprocess

import botocore.session
import pytest

from awscli.customizations import sessionmanager
from awscli.testutils import mock, unittest


class TestSessionManager(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock(botocore.session.Session)
        self.client = mock.Mock()
        self.region = 'us-west-2'
        self.profile = 'testProfile'
        self.endpoint_url = 'testUrl'
        self.client.meta.region_name = self.region
        self.client.meta.endpoint_url = self.endpoint_url
        self.session.create_client.return_value = self.client
        self.session.profile = self.profile
        self.caller = sessionmanager.StartSessionCaller(self.session)

        self.parsed_globals = mock.Mock()
        self.parsed_globals.profile = 'user_profile'

    def test_start_session_when_non_custom_start_session_fails(self):
        self.client.start_session.side_effect = Exception('some exception')
        params = {}
        with self.assertRaisesRegex(Exception, 'some exception'):
            self.caller.invoke('ssm', 'StartSession', params, mock.Mock())

    @mock.patch("awscli.customizations.sessionmanager.check_output")
    @mock.patch('awscli.customizations.sessionmanager.check_call')
    def test_start_session_success_scenario(
        self, mock_check_call, mock_check_output
    ):
        mock_check_output.return_value = "1.2.0.0\n"
        mock_check_call.return_value = 0

        start_session_params = {"Target": "i-123456789"}

        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }

        self.client.start_session.return_value = start_session_response

        rc = self.caller.invoke(
            'ssm', 'StartSession', start_session_params, self.parsed_globals
        )
        self.assertEqual(rc, 0)
        self.client.start_session.assert_called_with(**start_session_params)

        mock_check_output_list = mock_check_output.call_args[0]
        self.assertEqual(
            mock_check_output_list[0], ["session-manager-plugin", "--version"]
        )

        mock_check_call_list = mock_check_call.call_args[0][0]
        mock_check_call_list[1] = json.loads(mock_check_call_list[1])
        self.assertEqual(
            mock_check_call_list,
            [
                'session-manager-plugin',
                start_session_response,
                self.region,
                'StartSession',
                self.parsed_globals.profile,
                json.dumps(start_session_params),
                self.endpoint_url,
            ],
        )

    @mock.patch("awscli.customizations.sessionmanager.check_output")
    @mock.patch('awscli.customizations.sessionmanager.check_call')
    def test_start_session_when_check_call_fails(
        self, mock_check_call, mock_check_output
    ):
        mock_check_output.return_value = "1.2.0.0\n"
        mock_check_call.side_effect = OSError(errno.ENOENT, 'some error')

        start_session_params = {"Target": "i-123456789"}

        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }

        terminate_session_params = {"SessionId": "session-id"}

        self.client.start_session.return_value = start_session_response

        with self.assertRaises(ValueError):
            self.caller.invoke(
                'ssm',
                'StartSession',
                start_session_params,
                self.parsed_globals,
            )

            self.client.start_session.assert_called_with(
                **start_session_params
            )
            self.client.terminate_session.assert_called_with(
                **terminate_session_params
            )

            mock_check_call_list = mock_check_call.call_args[0][0]
            mock_check_call_list[1] = json.loads(mock_check_call_list[1])
            self.assertEqual(
                mock_check_call_list,
                [
                    'session-manager-plugin',
                    start_session_response,
                    self.region,
                    'StartSession',
                    self.parsed_globals.profile,
                    json.dumps(start_session_params),
                    self.endpoint_url,
                ],
            )

    @mock.patch('awscli.customizations.sessionmanager.check_call')
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_when_no_profile_is_passed(
        self, mock_check_output, mock_check_call
    ):
        mock_check_output.return_value = "1.2.500.0\n"
        self.session.profile = "session_profile"
        self.parsed_globals.profile = None
        mock_check_call.return_value = 0

        start_session_params = {"Target": "i-123456789"}

        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"

        self.client.start_session.return_value = start_session_response

        rc = self.caller.invoke(
            'ssm', 'StartSession', start_session_params, self.parsed_globals
        )
        self.assertEqual(rc, 0)
        self.client.start_session.assert_called_with(**start_session_params)
        mock_check_call_list = mock_check_call.call_args[0][0]
        self.assertEqual(
            mock_check_call_list,
            [
                "session-manager-plugin",
                ssm_env_name,
                self.region,
                "StartSession",
                "",
                json.dumps(start_session_params),
                self.endpoint_url,
            ],
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_with_env_variable_success_scenario(
        self, mock_check_output, mock_check_call
    ):
        mock_check_output.return_value = "1.2.500.0\n"
        mock_check_call.return_value = 0

        start_session_params = {"Target": "i-123456789"}

        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }
        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"

        self.client.start_session.return_value = start_session_response
        rc = self.caller.invoke(
            "ssm", "StartSession", start_session_params, self.parsed_globals
        )
        self.assertEqual(rc, 0)
        self.client.start_session.assert_called_with(**start_session_params)

        mock_check_output_list = mock_check_output.call_args[0]
        self.assertEqual(
            mock_check_output_list[0], ["session-manager-plugin", "--version"]
        )

        mock_check_call_list = mock_check_call.call_args[0][0]
        self.assertEqual(
            mock_check_call_list,
            [
                "session-manager-plugin",
                ssm_env_name,
                self.region,
                "StartSession",
                self.parsed_globals.profile,
                json.dumps(start_session_params),
                self.endpoint_url,
            ],
        )
        env_variable = mock_check_call.call_args[1]
        self.assertEqual(
            env_variable["env"][ssm_env_name],
            json.dumps(start_session_response),
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_when_check_output_fails(
        self, mock_check_output, mock_check_call
    ):
        mock_check_output.side_effect = subprocess.CalledProcessError(
            returncode=1, cmd="session-manager-plugin", output="some error"
        )

        start_session_params = {"Target": "i-123456789"}
        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }

        self.client.start_session.return_value = start_session_response
        with self.assertRaises(subprocess.CalledProcessError):
            self.caller.invoke(
                "ssm",
                "StartSession",
                start_session_params,
                self.parsed_globals,
            )

        self.client.start_session.assert_called_with(**start_session_params)
        self.client.terminate_session.assert_not_called()
        mock_check_output.assert_called_with(
            ["session-manager-plugin", "--version"], text=True
        )
        mock_check_call.assert_not_called()

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_when_response_not_json(
        self, mock_check_output, mock_check_call
    ):
        mock_check_output.return_value = "1.2.500.0\n"
        start_session_params = {"Target": "i-123456789"}
        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
            "para2": {"Not a json format"},
        }
        expected_env_value = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }

        ssm_env_name = "AWS_SSM_START_SESSION_RESPONSE"

        self.client.start_session.return_value = start_session_response
        rc = self.caller.invoke(
            "ssm", "StartSession", start_session_params, self.parsed_globals
        )
        self.assertEqual(rc, 0)
        self.client.start_session.assert_called_with(**start_session_params)

        mock_check_output_list = mock_check_output.call_args[0]
        self.assertEqual(
            mock_check_output_list[0], ["session-manager-plugin", "--version"]
        )

        mock_check_call_list = mock_check_call.call_args[0][0]
        self.assertEqual(
            mock_check_call_list,
            [
                "session-manager-plugin",
                ssm_env_name,
                self.region,
                "StartSession",
                self.parsed_globals.profile,
                json.dumps(start_session_params),
                self.endpoint_url,
            ],
        )
        env_variable = mock_check_call.call_args[1]
        self.assertEqual(
            env_variable["env"][ssm_env_name], json.dumps(expected_env_value)
        )

    @mock.patch("awscli.customizations.sessionmanager.check_call")
    @mock.patch("awscli.customizations.sessionmanager.check_output")
    def test_start_session_when_invalid_plugin_version(
        self, mock_check_output, mock_check_call
    ):
        mock_check_output.return_value = "InvalidVersion"

        start_session_params = {"Target": "i-123456789"}
        start_session_response = {
            "SessionId": "session-id",
            "TokenValue": "token-value",
            "StreamUrl": "stream-url",
        }

        self.client.start_session.return_value = start_session_response
        self.caller.invoke(
            "ssm", "StartSession", start_session_params, self.parsed_globals
        )
        self.client.start_session.assert_called_with(**start_session_params)
        self.client.terminate_session.assert_not_called()
        mock_check_output.assert_called_with(
            ["session-manager-plugin", "--version"], text=True
        )

        mock_check_call_list = mock_check_call.call_args[0][0]
        self.assertEqual(
            mock_check_call_list,
            [
                "session-manager-plugin",
                json.dumps(start_session_response),
                self.region,
                "StartSession",
                self.parsed_globals.profile,
                json.dumps(start_session_params),
                self.endpoint_url,
            ],
        )


class TestVersionRequirement:
    version_requirement = sessionmanager.VersionRequirement(
        min_version="1.2.497.0"
    )

    @pytest.mark.parametrize(
        "version, expected_result",
        [
            ("2.0.0.0", True),
            ("2.1", True),
            ("2", True),
            ("1.3.1.1", True),
            ("\r\n1. 3.1.1", True),
            ("1.3.1.0", True),
            ("1.3", True),
            ("1.2.498.1", True),
            ("1.2.498", True),
            ("1.2.497.1", True),
            ("1.2.497.0", False),
            ("1.2.497", False),
            ("1.2.1.1", False),
            ("1.2.1", False),
            ("1.2", False),
            ("1.1.1.0", False),
            ("1.1.1", False),
            ("1.0.497.0", False),
            ("1.0. 497.0\r\n", False),
            ("1", False),
            ("0.3.497.0", False),
        ],
    )
    def test_meets_requirement(self, version, expected_result):
        assert expected_result == self.version_requirement.meets_requirement(
            version
        )

    @pytest.mark.parametrize(
        "version, expected_result",
        [
            ("\r\n1.3.1.1", "1.3.1.1"),
            ("\r1.3 .1.1", "1.3.1.1"),
            ("1 .3.1.1", "1.3.1.1"),
            (" 1.3.1.1", "1.3.1.1"),
            ("1.3.1.1 ", "1.3.1.1"),
            (" 1.3.1.1 ", "1.3.1.1"),
            ("\n1.3.1.1 ", "1.3.1.1"),
            ("1.3.1.1\n", "1.3.1.1"),
            ("1.3\r\n.1.1", "1.3.1.1"),
            (" 1\r. 3", "1.3"),
            (" 1. 3. ", "1.3."),
            ("1.1.1\r\n", "1.1.1"),
            ("1\r", "1"),
        ],
    )
    def test_sanitize_plugin_version(self, version, expected_result):
        assert (
            expected_result
            == self.version_requirement._sanitize_plugin_version(version)
        )

    @pytest.mark.parametrize(
        "version, expected_result",
        [
            ("999.99999.99.9", True),
            ("2", True),
            ("1.1.1.1", True),
            ("1.1.1", True),
            ("1.1", True),
            ("1.1.1.1.1", False),
            ("1.1.1.1.0", False),
            ("1.1.1.a", False),
            ("1.a.1.1", False),
            ("1-1.1.1", False),
            ("1.1.", False),
            ("invalid_version", False),
        ],
    )
    def test_is_valid_version(self, version, expected_result):
        assert expected_result == self.version_requirement._is_valid_version(
            version
        )



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import contextlib
import json
import os
import pathlib
import re
import subprocess
import sys

import flit_core.buildapi
import pytest
from build_system.awscli_venv import AwsCliVenv
from build_system.constants import ArtifactType

from awscli.testutils import if_windows, skip_if_windows
from backends.build_system.constants import BIN_DIRNAME, PYTHON_EXE_NAME

ROOT_DIR = pathlib.Path(__file__).parents[4]


@contextlib.contextmanager
def cd(dirname):
    original = os.getcwd()
    os.chdir(dirname)
    try:
        yield
    finally:
        os.chdir(original)


@pytest.fixture(scope="session")
def venv_path(tmp_path_factory):
    return tmp_path_factory.mktemp("venv")


@pytest.fixture(scope="session")
def cli_venv(venv_path):
    venv = AwsCliVenv(venv_path)
    venv.create()
    return venv


class TestAwsCliVenv:
    def _normalize_dependency_entry(self, dep: str) -> str:
        dep = re.split("[<=>]", dep)[0]
        dep = dep.rstrip("<=>")
        dep = dep.lower()
        dep = dep.replace("-", "_")
        dep = dep.replace(".", "_")
        return dep

    def _normalize_dist_info_name(self, name: str) -> str:
        name = name.split("-")[0]
        name = name.lower()
        name = name.replace(".", "_")
        return name

    def _get_install_requires(self):
        with cd(ROOT_DIR):
            requires = flit_core.buildapi.get_requires_for_build_wheel()
        # Generation of the auto-complete index requires importing from the
        # awscli package and iterating over the commands from the clidriver. In
        # order to be able to do this, it requires all of the CLI's runtime
        # dependencies to be present to avoid import errors.
        dependency_block_re = re.compile(
            r"dependencies = \[([\s\S]+?)\]", re.MULTILINE
        )
        extract_dependencies_re = re.compile(r'"(.+)"')
        with open(ROOT_DIR / "pyproject.toml") as f:
            data = f.read()
        raw_dependencies = dependency_block_re.findall(data)[0]
        dependencies = extract_dependencies_re.findall(raw_dependencies)
        return dependencies + requires

    def _python_version(self) -> str:
        info = sys.version_info
        return f"python{info[0]}.{info[1]}"

    def _site_packages_dir(self, venv_path: pathlib.PurePath) -> str:
        site_path = [
            path
            for path in json.loads(
                subprocess.check_output(
                    [
                        venv_path / BIN_DIRNAME / PYTHON_EXE_NAME,
                        "-c",
                        "import site, json; print(json.dumps(site.getsitepackages()))",
                    ]
                )
                .decode()
                .strip()
            )
            if "site-packages" in path
        ][0]
        return site_path

    @skip_if_windows("Posix virtualenv")
    def test_create(self, tmp_path_factory):
        path = tmp_path_factory.mktemp("test_create")
        venv = AwsCliVenv(path)
        venv.create()

        venv_dirs = set(os.listdir(path))
        required_files = [
            "bin",
            "include",
            "pyvenv.cfg",
        ]
        for required_file in required_files:
            assert required_file in venv_dirs

    @if_windows("Windows virtualenv")
    def test_create_windows(self, tmp_path_factory):
        path = tmp_path_factory.mktemp("test_create")
        venv = AwsCliVenv(path)
        venv.create()

        venv_dirs = set(os.listdir(path))
        required_files = [
            "Scripts",
            "Include",
            "Lib",
            "pyvenv.cfg",
        ]
        for required_file in required_files:
            assert required_file in venv_dirs

    @skip_if_windows("Posix bootstrap")
    def test_bootstrap(self, cli_venv, venv_path):
        site_package_path = self._site_packages_dir(venv_path)

        prior_site_dir = set(os.listdir(site_package_path))
        cli_venv.bootstrap(
            ArtifactType.SYSTEM_SANDBOX.value, download_deps=False
        )
        post_site_dir = set(os.listdir(site_package_path))

        # Check that parent packages were installed
        added_packages = {
            self._normalize_dist_info_name(p)
            for p in post_site_dir - prior_site_dir
            if "dist-info" in p or "egg-info" in p
        }

        expected_packages = {
            self._normalize_dependency_entry(r)
            for r in self._get_install_requires()
        }
        missing_packages = expected_packages - added_packages

        assert missing_packages == set()

        # Check that the CLI is installed
        cli_path = (
            venv_path
            / "lib"
            / self._python_version()
            / "site-packages"
            / "awscli"
        )
        assert os.path.isdir(cli_path) is True

        # Make sure the ac.index got generated and injected correctly.
        ac_index_path = cli_path / "data" / "ac.index"
        assert os.path.exists(ac_index_path) is True

    @if_windows("Windows bootstrap")
    def test_bootstrap_windows(self, cli_venv, venv_path):
        site_package_path = self._site_packages_dir(venv_path)

        prior_site_dir = set(os.listdir(site_package_path))
        cli_venv.bootstrap(
            ArtifactType.SYSTEM_SANDBOX.value, download_deps=False
        )
        post_site_dir = set(os.listdir(site_package_path))

        # Check that parent packages were installed
        added_packages = {
            self._normalize_dist_info_name(p)
            for p in post_site_dir - prior_site_dir
            if "dist-info" in p
        }

        expected_packages = {
            self._normalize_dependency_entry(r)
            for r in self._get_install_requires()
        }
        missing_packages = expected_packages - added_packages

        assert missing_packages == set()

        # Check that the CLI is installed
        cli_path = venv_path / "Lib" / "site-packages" / "awscli"
        assert os.path.isdir(cli_path) is True

        # Make sure the ac.index got generated and injected correctly.
        ac_index_path = cli_path / "data" / "ac.index"
        assert os.path.exists(ac_index_path) is True

    @skip_if_windows("No bin dir on windows")
    def test_bin_dir(self, cli_venv, venv_path):
        assert cli_venv.bin_dir == os.path.join(venv_path, "bin")

    @if_windows("Scripts dir is only on windows")
    def test_scripts_dir(self, cli_venv, venv_path):
        assert cli_venv.bin_dir == os.path.join(venv_path, "Scripts")

    @skip_if_windows("Python binary location on posix")
    def test_python_exe(self, cli_venv, venv_path):
        assert cli_venv.python_exe == os.path.join(venv_path, "bin", "python")

    @if_windows("Python binary location on win")
    def test_python_exe_windows(self, cli_venv, venv_path):
        assert cli_venv.python_exe == os.path.join(
            venv_path, "Scripts", "python.exe"
        )



# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json

import colorama
from prompt_toolkit.application import Application
from prompt_toolkit.buffer import Buffer
from prompt_toolkit.input import create_pipe_input
from prompt_toolkit.key_binding import KeyPressEvent
from prompt_toolkit.output import DummyOutput

from awscli.compat import StringIO
from awscli.customizations.logs.ui import (
    COLOR_LIST,
    InputState,
    InteractivePrinter,
    InteractiveUI,
    Keyword,
    LiveTailBuffer,
    LiveTailKeyBindings,
    LiveTailLogEventsCollector,
    LiveTailSessionMetadata,
    OutputFormat,
    PrintOnlyPrinter,
    PrintOnlyUI,
)
from awscli.testutils import mock, unittest


class LiveTailSessionMetadataTest(unittest.TestCase):
    def setUp(self):
        self.session_metadata = LiveTailSessionMetadata()

    def test_metadata_update(self):
        metadata_update = {"sampled": True}
        self.session_metadata.update_metadata(metadata_update)

        self.assertEqual(
            metadata_update["sampled"], self.session_metadata.is_sampled
        )


class PrintOnlyPrinterTest(unittest.TestCase):
    def setUp(self):
        self.output = StringIO()
        self.log_events = []
        self.printer = PrintOnlyPrinter(self.output, self.log_events)

    def test_print_log_events(self):
        self.log_events.extend(
            ["First LogEvent", "Second LogEvent", "Third LogEvent"]
        )
        expected_msg = "\n".join(self.log_events) + "\n"

        self.printer._print_log_events()

        self.output.seek(0)
        self.assertEqual(expected_msg, self.output.read())
        self.assertEqual(0, len(self.log_events))

    def test_session_interrupt_while_printing(self):
        self.log_events.extend(
            ["First LogEvent", "Second LogEvent", "Third LogEvent"]
        )
        expected_msg = "\n".join(self.log_events) + "\n"

        self.printer.interrupt_session = True
        self.printer.run()

        self.output.seek(0)
        self.assertEqual(expected_msg, self.output.read())
        self.assertEqual(0, len(self.log_events))

    def test_exception_while_printing(self):
        self.log_events.extend(
            ["First LogEvent", "Second LogEvent", "Third LogEvent"]
        )
        self.printer._print_log_events = mock.MagicMock(
            side_effect=BrokenPipeError("BrokenPipe")
        )

        # No exception should be raised
        self.printer.run()


class PrintOnlyUITest(unittest.TestCase):
    def setUp(self):
        self.output = StringIO()
        self.log_events = []
        self.ui = PrintOnlyUI(self.output, self.log_events)

    def test_exit(self):
        self.ui.exit()

        self.assertEqual(True, self.ui._printer.interrupt_session)


class LiveTailLogEventsCollectorTest(unittest.TestCase):
    def setUp(self):
        self.output = StringIO()
        self.log_events = []
        self.response_stream = mock.Mock()
        self.ui = PrintOnlyUI(self.output, self.log_events)
        self.session_metadata = LiveTailSessionMetadata()
        self.log_events_collector = LiveTailLogEventsCollector(
            self.output,
            self.ui,
            self.response_stream,
            self.log_events,
            self.session_metadata,
        )

    def test_log_event_collection(self):
        updates = [
            {"sessionStart": "The session is started"},
            {
                "sessionUpdate": {
                    "sessionMetadata": {"sampled": False},
                    "sessionResults": [
                        {"message": "LogEvent1"},
                        {"message": "LogEvent2"},
                    ],
                }
            },
        ]
        self.response_stream.__iter__ = mock.MagicMock(
            return_value=iter(updates)
        )

        self.log_events_collector._collect_log_events()

        self.assertEqual(2, len(self.log_events))
        self.assertEqual(["LogEvent1", "LogEvent2"], self.log_events)
        self.assertIsNone(self.log_events_collector._exception)

    def test_log_event_collection_with_unexpected_update(self):
        updates = [{"unexpectedUpdate": "The session is started"}]
        self.response_stream.extend(updates)
        self.ui.exit = mock.MagicMock()

        self.log_events_collector._collect_log_events()

        self.assertIsNotNone(self.log_events_collector._exception)

    def test_stop_with_exception(self):
        exception_message = "SessionStreamingException"
        self.log_events_collector._exception = Exception(exception_message)

        self.log_events_collector.stop()

        self.output.seek(0)
        self.assertEqual(exception_message + "\n", self.output.read())

    def test_stop_without_exception(self):
        self.response_stream.close = mock.MagicMock()

        self.log_events_collector.stop()
        self.response_stream.close.assert_not_called()


class KeywordTest(unittest.TestCase):
    def setUp(self) -> None:
        self.keyword = None

    def test_text(self):
        text = "KEYWORD"
        self.keyword = Keyword(text, COLOR_LIST[0])

        self.assertEqual(text.lower(), self.keyword.text)

    def test_color_protected_keywords(self):
        text = "INFO"
        self.keyword = Keyword(text)

        self.assertEqual(text.lower(), self.keyword.text)
        self.assertEqual(colorama.Fore.GREEN, self.keyword._color)

        text = "ERROR"
        self.keyword = Keyword(text)

        self.assertEqual(text.lower(), self.keyword.text)
        self.assertEqual(colorama.Fore.RED, self.keyword._color)

        text = "WARN"
        self.keyword = Keyword(text)

        self.assertEqual(text.lower(), self.keyword.text)
        self.assertEqual(colorama.Fore.YELLOW, self.keyword._color)

    def test_color_unprotected_keywords(self):
        text = "random"
        keyword_1 = Keyword(text, COLOR_LIST[0])

        self.assertEqual(text.lower(), keyword_1.text)
        self.assertTrue(keyword_1._color in COLOR_LIST)

        text = "something_else_random"
        keyword_2 = Keyword(text, COLOR_LIST[1])

        self.assertEqual(text.lower(), keyword_2.text)
        self.assertTrue(keyword_2._color in COLOR_LIST)

        self.assertNotEqual(keyword_1._color, keyword_2._color)

    def test_string_to_print(self):
        text = "INFO"
        self.keyword = Keyword(text)

        self.assertEqual(
            colorama.Fore.GREEN
            + text.lower()
            + colorama.Style.RESET_ALL
            + ": 0",
            self.keyword.get_string_to_print(),
        )

    def test_add_color_to_string(self):
        text = "INFO"
        log_event = "This is an INFO log"
        self.keyword = Keyword(text)

        colored_log_event = self.keyword._add_color_to_string(
            log_event, 11, 15
        )

        self.assertEqual(
            "This is an "
            + colorama.Fore.GREEN
            + text
            + colorama.Style.RESET_ALL
            + " log",
            colored_log_event,
        )

    def test_remove_color_from_string(self):
        text = "INFO"
        log_event = "This is an INFO log"
        self.keyword = Keyword(text)

        uncolored_log_event = self.keyword._remove_color_from_string(
            log_event, 15, 19
        )

        self.assertEqual(log_event, uncolored_log_event)

    def test_highlight(self):
        text = "INFO"
        log_event = "This is an INFO log with info occurring a lot of times. Because we are testing INFO."
        self.keyword = Keyword(text)

        log_event_after_highlighting = self.keyword.highlight(log_event)

        self.assertEqual(
            "This is an "
            + colorama.Fore.GREEN
            + text
            + colorama.Style.RESET_ALL
            + " log with "
            + colorama.Fore.GREEN
            + text.lower()
            + colorama.Style.RESET_ALL
            + " occurring a lot of times. Because we are testing "
            + colorama.Fore.GREEN
            + text
            + colorama.Style.RESET_ALL
            + ".",
            log_event_after_highlighting,
        )


class InteractivePrinterTest(unittest.TestCase):
    def setUp(self) -> None:
        self.ui = mock.Mock(spec=InteractiveUI)
        self.output = LiveTailBuffer()
        self.log_events = []
        self.session_metadata = LiveTailSessionMetadata()
        self.keyword_highlight = {}
        self.printer = InteractivePrinter(
            self.ui,
            self.output,
            self.log_events,
            self.session_metadata,
            self.keyword_highlight,
        )

    def test_color_log_event(self):
        log_event = "[INFO] this is a log event with KEYWORD"
        keyword = Keyword("KEYWORD", COLOR_LIST[0])
        self.keyword_highlight[keyword.text] = keyword

        colored_log_event = self.printer._color_log_event(log_event)

        self.assertEqual(
            "["
            + colorama.Fore.GREEN
            + "INFO"
            + colorama.Style.RESET_ALL
            + "] this is a log event with "
            + COLOR_LIST[0]
            + "KEYWORD"
            + colorama.Style.RESET_ALL,
            colored_log_event,
        )

    def test_format_log_event_json(self):
        log_event = '{ "message": "This is a LogEvent", "logger": "self" }'
        json_log_event = json.dumps(json.loads(log_event), indent=4)

        formatted_log_event = self.printer._format_log_event(log_event)

        self.assertEqual(json_log_event, formatted_log_event)

    def test_format_log_event_not_json(self):
        log_event = "this is a log event"

        formatted_log_event = self.printer._format_log_event(log_event)

        self.assertEqual(log_event, formatted_log_event)

    def test_toggle_formatting(self):
        self.printer.toggle_formatting()

        self.assertEqual(OutputFormat.PLAIN_TEXT, self.printer.output_format)

    def test_print_log_events(self):
        self.session_metadata.update_metadata({"sampled": True})
        log_event_1 = "[ERROR] in todays log."
        log_event_2 = '{ "severity": "INFO", "message": "This keyword will be highlighted." }'
        self.log_events.extend([log_event_1, log_event_2])
        keyword = Keyword("keyword", COLOR_LIST[0])
        self.keyword_highlight[keyword.text] = keyword
        expected_output = "{}\n{}\n".format(
            self.printer._color_log_event(
                self.printer._format_log_event(log_event_1)
            ),
            self.printer._color_log_event(
                self.printer._format_log_event(log_event_2)
            ),
        )

        self.printer._print_log_events()

        self.assertEqual(expected_output, self.output.text)
        self.assertEqual(2, self.printer.log_events_displayed)
        self.assertEqual(True, self.printer.is_sampled)
        self.assertEqual(0, len(self.log_events))
        self.ui.update_metadata.assert_called_once_with()


class LiveTailBufferTest(unittest.TestCase):
    def setUp(self) -> None:
        self.buffer = LiveTailBuffer()
        self.layout = mock.MagicMock()

    def test_add_text_when_not_in_focus(self):
        self.layout.has_focus.return_value = False

        self.buffer.add_text("random text")

        self.assertEqual("random text", self.buffer.text)
        self.assertEqual(len(self.buffer.text), self.buffer.cursor_position)


class LiveTailKeyBindingsTest(unittest.TestCase):
    def setUp(self) -> None:
        self.ui = mock.Mock(InteractiveUI)
        self.prompt_buffer = Buffer()
        self.output_buffer = Buffer()
        self.ui._application = mock.Mock(Application)
        self.keyword_highlight = {}
        self.key_bindings = LiveTailKeyBindings(
            self.ui,
            self.prompt_buffer,
            self.output_buffer,
            self.keyword_highlight,
        )

    def test_reset(self):
        event = mock.Mock()
        event.app = self.ui._application
        self.key_bindings._reset(event)

        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)
        self.assertEqual("", self.key_bindings._input_buffer.text)
        self.ui.update_bottom_toolbar.assert_called_once_with(
            self.ui.get_instructions()
        )
        self.ui.update_quit_button.assert_called_once_with(
            self.ui.EXIT_SESSION
        )
        self.ui._application.invalidate.assert_called_once_with()

    def test_any_key_binding(self):
        self.key_bindings.get_bindings_for_keys("a")[0].handler(
            mock.Mock(spec=KeyPressEvent)
        )

        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)

        # if input is in highlight mode, text can be added to the buffer
        self.key_bindings.get_bindings_for_keys("h")[-1].handler(
            mock.Mock(spec=KeyPressEvent)
        )
        keypress_a = mock.Mock(spec=KeyPressEvent)
        keypress_a.data = "a"
        self.key_bindings.get_bindings_for_keys("a")[1].handler(keypress_a)

        self.assertEqual("a", self.key_bindings._input_buffer.text)

        # adding text buffer to the keyword
        print(self.key_bindings.bindings)
        self.key_bindings.bindings[7].handler(mock.Mock(spec=KeyPressEvent))

        # moving the input to clear mode
        keypress_c = mock.Mock(spec=KeyPressEvent)
        keypress_c.app = self.ui._application
        self.key_bindings.get_bindings_for_keys("c")[-1].handler(keypress_c)
        keypress_1 = mock.Mock(spec=KeyPressEvent)
        keypress_1.data = "1"
        self.key_bindings.get_bindings_for_keys("1")[2].handler(keypress_1)

        self.assertEqual(0, len(self.keyword_highlight))
        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)
        self.assertEqual("", self.key_bindings._input_buffer.text)
        self.ui.update_bottom_toolbar.assert_called_with(
            self.ui.get_instructions()
        )
        self.ui.update_quit_button.assert_called_with(self.ui.EXIT_SESSION)
        self.ui._application.invalidate.assert_called_with()

    def test_h_key_binding(self):
        keypress_h = mock.Mock(spec=KeyPressEvent)
        keypress_h.app = self.ui._application
        self.key_bindings.get_bindings_for_keys("h")[-1].handler(keypress_h)

        self.assertEqual(InputState.HIGHLIGHT, self.key_bindings._input_state)
        self.ui.update_bottom_toolbar.assert_called_once_with(
            self.ui.HIGHLIGHT_INSTRUCTIONS
        )
        self.ui.update_quit_button.assert_called_once_with(
            self.ui.EXIT_HIGHLIGHT
        )
        self.ui._application.invalidate.assert_called_once_with()

        self.key_bindings._reset(keypress_h)

        self.keyword_highlight["text1"] = Keyword("text1", COLOR_LIST[0])
        self.keyword_highlight["text2"] = Keyword("text2", COLOR_LIST[1])
        self.keyword_highlight["text3"] = Keyword("text3", COLOR_LIST[2])
        self.keyword_highlight["text4"] = Keyword("text4", COLOR_LIST[3])
        self.keyword_highlight["text5"] = Keyword("text5", COLOR_LIST[4])

        # Filter condition for keybinding is False if there are 5 or greater keywords
        self.assertFalse(
            self.key_bindings.get_bindings_for_keys("h")[-1].filter()
        )

    def test_c_key_binding(self):
        keypress_c = mock.Mock(spec=KeyPressEvent)
        keypress_c.app = self.ui._application

        # Nothing is executed unless there's a keyword to clean
        self.assertFalse(
            self.key_bindings.get_bindings_for_keys("c")[-1].filter()
        )
        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)

        keyword = Keyword("text", COLOR_LIST[0])
        self.keyword_highlight[keyword.text] = keyword

        self.key_bindings.get_bindings_for_keys("c")[-1].handler(keypress_c)

        self.assertEqual(InputState.CLEAR, self.key_bindings._input_state)
        self.ui.update_bottom_toolbar.assert_called_once_with(
            "1: text    ENTER: All"
        )
        self.ui.update_quit_button.assert_called_once_with(self.ui.EXIT_CLEAR)
        self.ui._application.invalidate.assert_called_once_with()

    def test_t_key_binding(self):
        self.key_bindings.get_bindings_for_keys("t")[-1].handler(
            mock.Mock(spec=KeyPressEvent)
        )

        self.ui.toggle_formatting.assert_called_once_with()
        self.ui.get_instructions.assert_called_once_with()
        self.ui.update_bottom_toolbar.assert_called_once_with(
            self.ui.get_instructions()
        )

    def test_q_key_binding(self):
        keypress_q = mock.Mock(spec=KeyPressEvent)
        keypress_q.app = self.ui._application
        self.ui._application.layout = mock.Mock()
        self.ui._application.layout.has_focus.return_value = False

        self.key_bindings.get_bindings_for_keys("q")[-1].handler(keypress_q)

        self.ui.handle_scrolling.assert_called_once_with(False)
        self.ui._application.layout.focus.assert_called_once_with(
            self.prompt_buffer
        )

    def test_enter_key_binding(self):
        # enter pressed when input disabled
        keypress_enter = mock.Mock(spec=KeyPressEvent)
        keypress_enter.app = self.ui._application
        self.key_bindings.bindings[7].handler(keypress_enter)

        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)
        self.assertEqual("", self.key_bindings._input_buffer.text)
        self.ui.update_bottom_toolbar.assert_called_once_with(
            self.ui.get_instructions()
        )
        self.ui.update_quit_button.assert_called_once_with(
            self.ui.EXIT_SESSION
        )
        self.ui._application.invalidate.assert_called_once_with()

        # going to highlight mode
        keypress_h = mock.Mock(spec=KeyPressEvent)
        keypress_h.app = self.ui._application
        self.key_bindings.get_bindings_for_keys("h")[-1].handler(keypress_h)
        self.key_bindings._input_buffer.text = "keyword"

        self.key_bindings.bindings[7].handler(keypress_enter)

        # assert keywords added to highlight
        self.assertTrue("keyword" in self.keyword_highlight)
        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)
        self.assertEqual("", self.key_bindings._input_buffer.text)
        self.ui.update_bottom_toolbar.assert_called_with(
            self.ui.get_instructions()
        )
        self.ui.update_quit_button.assert_called_with(self.ui.EXIT_SESSION)
        self.ui._application.invalidate.assert_called_with()

        # going to clear mode
        keypress_c = mock.Mock(spec=KeyPressEvent)
        keypress_c.app = self.ui._application
        self.key_bindings.get_bindings_for_keys("c")[-1].handler(keypress_c)
        self.key_bindings.bindings[8].handler(keypress_enter)

        # assert keywords cleared
        self.assertEqual(0, len(self.keyword_highlight))
        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)
        self.assertEqual("", self.key_bindings._input_buffer.text)
        self.ui.update_bottom_toolbar.assert_called_with(
            self.ui.get_instructions()
        )
        self.ui.update_quit_button.assert_called_with(self.ui.EXIT_SESSION)
        self.ui._application.invalidate.assert_called_with()

    def test_backspace_key_binding(self):
        self.prompt_buffer.text = "text"

        self.key_bindings.bindings[9].handler(mock.Mock(spec=KeyPressEvent))

        self.assertEqual("tex", self.prompt_buffer.text)

    def test_escape_key_binding(self):
        escape_event = mock.Mock(spec=KeyPressEvent)
        escape_event.app = self.ui._application

        self.key_bindings.bindings[11].handler(escape_event)

        self.ui._application.exit.assert_called_once_with()

        # if in highlight mode, escape gets you out of highlight
        self.key_bindings._input_state = InputState.HIGHLIGHT

        self.key_bindings.bindings[10].handler(escape_event)

        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)

        # if in clear mode, escape gets you out of clear
        self.key_bindings._input_state = InputState.CLEAR

        self.key_bindings.bindings[10].handler(escape_event)

        self.assertEqual(InputState.DISABLED, self.key_bindings._input_state)

    def test_ctrl_c_key_binding(self):
        escape_event = mock.Mock(spec=KeyPressEvent)
        escape_event.app = self.ui._application

        self.key_bindings.bindings[12].handler(escape_event)

        self.ui._application.exit.assert_called_once_with()


class InteractiveUITest(unittest.IsolatedAsyncioTestCase):
    def setUp(self) -> None:
        self.output = StringIO
        self.log_events = []
        self.session_metadata = LiveTailSessionMetadata()
        self.ui = InteractiveUI(
            self.log_events,
            self.session_metadata,
            app_output=DummyOutput(),
            app_input=create_pipe_input(),
        )

    def test_update_toolbar(self):
        text = "new text"

        self.ui.update_bottom_toolbar(text)

        self.assertEqual(text, self.ui._bottom_toolbar.text)

    def test_update_quit_button(self):
        text = "new text"

        self.ui.update_quit_button(text)

        self.assertEqual(text, self.ui._quit_button.text)

    def test_metadata_update(self):
        keyword = Keyword("text", COLOR_LIST[0])
        self.ui._keywords_to_highlight[keyword.text] = keyword

        self.ui.update_metadata()

        self.assertTrue(keyword.text in self.ui._metadata.text.value)

    def test_get_instructions(self):
        instructions = self.ui.INSTRUCTIONS.format(
            colorama.Fore.GREEN
            + OutputFormat.JSON.value
            + colorama.Style.RESET_ALL,
            OutputFormat.PLAIN_TEXT.value,
        )
        self.assertEqual(instructions, self.ui.get_instructions().value)

        self.ui._log_events_printer.toggle_formatting()
        instructions = self.ui.INSTRUCTIONS.format(
            OutputFormat.JSON.value,
            colorama.Fore.GREEN
            + OutputFormat.PLAIN_TEXT.value
            + colorama.Style.RESET_ALL,
        )
        self.assertEqual(instructions, self.ui.get_instructions().value)

        self.ui._is_scroll_active = True
        instructions = (
            self.ui.INSTRUCTIONS.format(
                OutputFormat.JSON.value,
                colorama.Fore.GREEN
                + OutputFormat.PLAIN_TEXT.value
                + colorama.Style.RESET_ALL,
            )
            + "    q: Scroll to latest"
        )
        self.assertEqual(instructions, self.ui.get_instructions().value)

    def test_handle_scrolling(self):
        self.ui.get_instructions = mock.MagicMock(return_value="instructions")
        self.ui.update_bottom_toolbar = mock.MagicMock()

        self.ui.handle_scrolling(True)
        self.ui.update_bottom_toolbar.assert_called_once_with(
            self.ui.get_instructions()
        )

        self.ui._key_bindings._input_state = InputState.HIGHLIGHT
        self.ui.handle_scrolling(True)
        self.ui.update_bottom_toolbar.assert_called_once_with(
            self.ui.get_instructions()
        )

        self.ui._output._pause_buffer.text = "random text"
        self.ui._trim_buffer = mock.MagicMock()
        self.ui._application.create_background_task = mock.MagicMock()
        self.ui.handle_scrolling(False)
        self.assertEqual("random text", self.ui._output.text)
        self.assertEqual(
            len(self.ui._output.text), self.ui._output.cursor_position
        )
        self.assertEqual("", self.ui._output._pause_buffer.text)
        self.ui._trim_buffer.assert_called_once_with(self.ui._output)

    async def test_highlight_term_in_buffer(self):
        keyword = Keyword("INFO")
        log_event = "This is an INFO log"
        self.ui._output.text = log_event
        self.ui._output.pause_buffer.text = log_event

        await self.ui.highlight_term_in_buffer(keyword)

        self.assertEqual(keyword.highlight(log_event), self.ui._output.text)
        self.assertEqual(
            keyword.highlight(log_event), self.ui._output._pause_buffer.text
        )

    async def test_remove_highlight_term_in_buffer(self):
        keyword = Keyword("INFO")
        log_event = "This is an INFO log"
        colored_log_event = keyword.highlight(log_event)
        self.ui._output.text = colored_log_event
        self.ui._output.pause_buffer.text = colored_log_event

        await self.ui.remove_term_from_buffer(keyword)

        self.assertEqual(log_event, self.ui._output.text)
        self.assertEqual(log_event, self.ui._output._pause_buffer.text)

    async def test_trim_buffer(self):
        self.ui._MAX_LINE_COUNT = 1
        log_event = Buffer()
        log_event.text += "text\n"
        log_event.text += "text\n"

        log_event_after_trim = await self.ui._trim_buffer(log_event)

        self.assertEqual("text\n", log_event_after_trim)

    def test_exit(self):
        self.ui._application.exit = mock.MagicMock()

        self.ui.exit()

        self.ui._application.exit.assert_called_once_with()



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestGetLogEvents(BaseAWSCommandParamsTest):
    prefix = 'logs get-log-events'

    # TODO: Get rid of this test once we do the following:
    # 1) Add support for pagination operations that return repeat next
    #    tokens to indicate that pagination has ended. Currently an
    #    error is thrown for this.
    # 2) We add a pagination model for GetLogEvents
    def test_cannot_paginate(self):
        cmdline = self.prefix
        cmdline += ' --log-group-name foo'
        cmdline += ' --log-stream-name bar'
        cmdline += ' --page-size 5'
        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=252)
        self.assertIn('Unknown options', stderr)
        self.assertIn('--page-size', stderr)



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestModifyInstanceAttribute(BaseAWSCommandParamsTest):
    prefix = 'ec2 modify-image-attribute'

    def test_one(self):
        cmdline = self.prefix
        cmdline += ' --image-id ami-d00dbeef'
        cmdline += ' --operation-type add'
        cmdline += ' --user-ids 0123456789012'
        result = {
            'ImageId': 'ami-d00dbeef',
            'OperationType': 'add',
            'UserIds': ['0123456789012'],
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_two(self):
        cmdline = self.prefix
        cmdline += ' --image-id ami-d00dbeef'
        cmdline += (
            ' --launch-permission {"Add":[{"UserId":"123456789012"}],'
            '"Remove":[{"Group":"all"}]}'
        )
        result = {
            'ImageId': 'ami-d00dbeef',
            'LaunchPermission': {
                'Add': [{'UserId': '123456789012'}],
                'Remove': [{'Group': 'all'}],
            },
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_assert_error_in_bad_json_path(self):
        cmdline = self.prefix
        cmdline += ' --image-id ami-d00dbeef'
        cmdline += ' --launch-permission THISISNOTJSON'
        # The arg name should be in the error message.
        self.assert_params_for_cmd(
            cmdline, expected_rc=252, stderr_contains='launch-permission'
        )


if __name__ == "__main__":
    unittest.main()



from awscli import __version__ as awscli_version
from tests import CLIRunner
from tests.functional.botocore.test_useragent import (
    get_captured_ua_strings,
    parse_registered_feature_ids,
)


def assert_expected_user_agent(result, service, operation):
    ua_string = result.aws_requests[0].http_requests[0].headers['User-Agent']
    assert ua_string.startswith(f'aws-cli/{awscli_version}')
    assert ' md/installer#' in ua_string
    assert ' md/awscrt#' in ua_string
    assert ' md/arch#' in ua_string
    assert ' md/prompt#off' in ua_string
    assert f' md/command#{service}.{operation}' in ua_string
    assert ' ua/2.1 ' in ua_string
    assert ' os/' in ua_string
    assert ' lang/python' in ua_string
    assert ' cfg/' in ua_string
    assert f' md/command#{service}.{operation}' in ua_string


def test_basic_user_agent():
    cli_runner = CLIRunner()
    service = 'sts'
    operation = 'get-caller-identity'
    result = cli_runner.run([service, operation])
    assert_expected_user_agent(result, service, operation)


def test_user_agent_for_customization():
    cli_runner = CLIRunner()
    service = 's3'
    operation = 'ls'
    result = cli_runner.run([service, operation])
    assert_expected_user_agent(result, service, operation)
    ua_string = result.aws_requests[0].http_requests[0].headers['User-Agent']
    feature_list = parse_registered_feature_ids(ua_string)
    assert 'C' in feature_list



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import sys
from argparse import Namespace
from socket import timeout

from awscli.customizations.codedeploy.systems import (
    RHEL,
    System,
    Ubuntu,
    Windows,
)
from awscli.customizations.codedeploy.utils import (
    MAX_INSTANCE_NAME_LENGTH,
    MAX_TAG_KEY_LENGTH,
    MAX_TAG_VALUE_LENGTH,
    MAX_TAGS_PER_INSTANCE,
    validate_iam_user_arn,
    validate_instance,
    validate_instance_name,
    validate_region,
    validate_s3_location,
    validate_tags,
)
from awscli.customizations.exceptions import (
    ConfigurationError,
    ParamValidationError,
)
from awscli.testutils import mock, unittest


class TestUtils(unittest.TestCase):
    def setUp(self):
        self.iam_user_arn = 'arn:aws:iam::012345678912:user/AWS/CodeDeploy/foo'
        self.region = 'us-east-1'
        self.arg_name = 's3-location'
        self.bucket = 'bucket'
        self.key = 'key'

        self.system_patcher = mock.patch('platform.system')
        self.system = self.system_patcher.start()
        self.system.return_value = 'Linux'

        self.linux_distribution_patcher = mock.patch(
            'awscli.compat.linux_distribution'
        )
        self.linux_distribution = self.linux_distribution_patcher.start()
        self.linux_distribution.return_value = ('Ubuntu', '', '')

        self.urlopen_patcher = mock.patch(
            'awscli.customizations.codedeploy.utils.urlopen'
        )
        self.urlopen = self.urlopen_patcher.start()
        self.urlopen.side_effect = timeout('Not EC2 instance')

        self.globals = mock.MagicMock()
        self.session = mock.MagicMock()
        self.params = Namespace()
        self.params.session = self.session

    def tearDown(self):
        self.system_patcher.stop()
        self.linux_distribution_patcher.stop()
        self.urlopen_patcher.stop()

    def test_validate_region_returns_global_region(self):
        self.globals.region = self.region
        self.session.get_config_variable.return_value = None
        validate_region(self.params, self.globals)
        self.assertIn('region', self.params)
        self.assertEqual(self.region, self.params.region)

    def test_validate_region_returns_session_region(self):
        self.globals.region = None
        self.session.get_config_variable.return_value = self.region
        validate_region(self.params, self.globals)
        self.assertIn('region', self.params)
        self.assertEqual(self.region, self.params.region)

    def test_validate_region_throws_on_no_region(self):
        self.globals.region = None
        self.session.get_config_variable.return_value = None
        error_msg = 'Region not specified.'
        with self.assertRaisesRegex(ConfigurationError, error_msg):
            validate_region(self.params, self.globals)

    def test_validate_instance_name(self):
        instance_name = 'instance-name'
        self.params.instance_name = instance_name
        validate_instance_name(self.params)

    def test_validate_instance_name_throws_on_invalid_characters(self):
        self.params.instance_name = '!#$%^&*()<>/?;:[{]}'
        error_msg = 'Instance name contains invalid characters.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_instance_name(self.params)

    def test_validate_instance_name_throws_on_i_dash(self):
        self.params.instance_name = 'i-instance'
        error_msg = "Instance name cannot start with 'i-'."
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_instance_name(self.params)

    def test_validate_instance_name_throws_on_long_name(self):
        self.params.instance_name = (
            '01234567890123456789012345678901234567890123456789'
            '012345678901234567890123456789012345678901234567891'
        )
        error_msg = f'Instance name cannot be longer than {MAX_INSTANCE_NAME_LENGTH} characters.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_instance_name(self.params)

    def test_validate_tags_throws_on_too_many_tags(self):
        self.params.tags = [
            {'Key': 'k' + str(x), 'Value': 'v' + str(x)} for x in range(11)
        ]
        error_msg = f'Instances can only have a maximum of {MAX_TAGS_PER_INSTANCE} tags.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_tags(self.params)

    def test_validate_tags_throws_on_max_key_not_accepted(self):
        key = 'k' * 128
        self.params.tags = [{'Key': key, 'Value': 'v1'}]
        validate_tags(self.params)

    def test_validate_tags_throws_on_long_key(self):
        key = 'k' * 129
        self.params.tags = [{'Key': key, 'Value': 'v1'}]
        error_msg = (
            f'Tag Key cannot be longer than {MAX_TAG_KEY_LENGTH} characters.'
        )
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_tags(self.params)

    def test_validate_tags_throws_on_max_value_not_accepted(self):
        value = 'v' * 256
        self.params.tags = [{'Key': 'k1', 'Value': value}]
        validate_tags(self.params)

    def test_validate_tags_throws_on_long_value(self):
        value = 'v' * 257
        self.params.tags = [{'Key': 'k1', 'Value': value}]
        error_msg = f'Tag Value cannot be longer than {MAX_TAG_VALUE_LENGTH} characters.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_tags(self.params)

    def test_validate_iam_user_arn(self):
        self.params.iam_user_arn = self.iam_user_arn
        validate_iam_user_arn(self.params)

    def test_validate_iam_user_arn_throws_on_invalid_arn_pattern(self):
        self.params.iam_user_arn = 'invalid-arn-pattern'
        error_msg = 'Invalid IAM user ARN.'
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_iam_user_arn(self.params)

    def test_validate_instance_ubuntu(self):
        self.urlopen.side_effect = timeout('Not EC2 instance')
        self.system.return_value = 'Linux'
        self.linux_distribution.return_value = ('Ubuntu', None, None)
        self.params.session = self.session
        self.params.region = self.region
        validate_instance(self.params)
        self.assertIn('system', self.params)
        self.assertTrue(isinstance(self.params.system, Ubuntu))

    def test_validate_instance_rhel(self):
        self.urlopen.side_effect = timeout('Not EC2 instance')
        self.system.return_value = 'Linux'
        self.linux_distribution.return_value = (
            'Red Hat Enterprise Linux Server',
            None,
            None,
        )
        self.params.session = self.session
        self.params.region = self.region
        validate_instance(self.params)
        self.assertIn('system', self.params)
        self.assertTrue(isinstance(self.params.system, RHEL))

    def test_validate_instance_windows(self):
        self.urlopen.side_effect = timeout('Not EC2 instance')
        self.system.return_value = 'Windows'
        self.params.session = self.session
        self.params.region = self.region
        validate_instance(self.params)
        self.assertIn('system', self.params)
        self.assertTrue(isinstance(self.params.system, Windows))

    def test_validate_instance_throws_on_unsupported_system(self):
        self.system.return_value = 'Unsupported'
        with self.assertRaisesRegex(
            RuntimeError, System.UNSUPPORTED_SYSTEM_MSG
        ):
            validate_instance(self.params)

    def test_validate_instance_throws_on_ec2_instance(self):
        self.params.session = self.session
        self.params.region = self.region
        self.urlopen.side_effect = None
        with self.assertRaisesRegex(
            RuntimeError, 'Amazon EC2 instances are not supported.'
        ):
            validate_instance(self.params)

    def test_validate_s3_location_returns_bucket_key(self):
        self.params.s3_location = f's3://{self.bucket}/{self.key}'
        validate_s3_location(self.params, self.arg_name)
        self.assertIn('bucket', self.params)
        self.assertEqual(self.bucket, self.params.bucket)
        self.assertIn('key', self.params)
        self.assertEqual(self.key, self.params.key)

    def test_validate_s3_location_not_present(self):
        validate_s3_location(self.params, 'unknown')
        self.assertNotIn('bucket', self.params)
        self.assertNotIn('key', self.params)

    def test_validate_s3_location_throws_on_invalid_location(self):
        self.params.s3_location = 'invalid-s3-location'
        error_msg = (
            f'--{self.arg_name} must specify the Amazon S3 URL format as '
            's3://<bucket>/<key>.'
        )
        with self.assertRaisesRegex(ParamValidationError, error_msg):
            validate_s3_location(self.params, self.arg_name)


if __name__ == "__main__":
    unittest.main()



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.session import Session

from awscli.customizations.wizard import app, core, factory
from awscli.testutils import mock, unittest


class TestCanCreateWizardComponents(unittest.TestCase):
    def test_can_create_default_v1_runner(self):
        session = mock.Mock(spec=Session)
        runner = factory.create_default_wizard_v1_runner(session)
        self.assertIsInstance(runner, core.Runner)

    def test_can_create_default_v2_runner(self):
        session = mock.Mock(spec=Session)
        runner = factory.create_default_wizard_v2_runner(session)
        self.assertIsInstance(runner, app.WizardAppRunner)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from botocore import UNSIGNED
from botocore.session import get_session

from awscli.customizations import globalargs
from awscli.customizations.exceptions import ParamValidationError
from awscli.testutils import mock, unittest


class FakeParsedArgs:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def __getattr__(self, arg):
        return None


class FakeSession:
    def __init__(self, session_vars=None, config_file_vars=None):
        if session_vars is None:
            session_vars = {}
        self.session_var_map = session_vars
        if config_file_vars is None:
            config_file_vars = {}
        self.config_file_vars = config_file_vars

    def get_config_variable(
        self, name, methods=('env', 'config'), default=None
    ):
        value = None
        config_name, envvar_name = self.session_var_map[name]
        if methods is not None:
            if 'env' in methods and value is None:
                value = os.environ.get(envvar_name)
            if 'config' in methods and value is None:
                value = self.config_file_vars.get(config_name)
        else:
            value = default
        return value


class TestGlobalArgsCustomization(unittest.TestCase):
    def test_parse_query(self):
        parsed_args = FakeParsedArgs(query='foo.bar')
        globalargs.resolve_types(parsed_args)
        # Assert that it looks like a jmespath parsed expression.
        self.assertFalse(isinstance(parsed_args.query, str))
        self.assertTrue(hasattr(parsed_args.query, 'search'))

    def test_parse_query_error_message(self):
        # Invalid jmespath expression.
        parsed_args = FakeParsedArgs(query='foo.bar.')
        with self.assertRaises(ParamValidationError):
            globalargs.resolve_types(parsed_args)
            globalargs.resolve_types('query')

    def test_parse_verify_ssl_default_value(self):
        with mock.patch('os.environ', {}):
            parsed_args = FakeParsedArgs(verify_ssl=True, ca_bundle=None)
            session_var_map = {'ca_bundle': ('ca_bundle', 'AWS_CA_BUNDLE')}
            session = FakeSession(session_vars=session_var_map)
            globalargs.resolve_verify_ssl(parsed_args, session)
            # None, so that botocore can apply it's default logic.
            self.assertIsNone(parsed_args.verify_ssl)

    def test_parse_verify_ssl_verify_turned_off(self):
        with mock.patch('os.environ', {}):
            parsed_args = FakeParsedArgs(verify_ssl=False, ca_bundle=None)
            session_var_map = {'ca_bundle': ('ca_bundle', 'AWS_CA_BUNDLE')}
            session = FakeSession(session_vars=session_var_map)
            globalargs.resolve_verify_ssl(parsed_args, session)
            self.assertFalse(parsed_args.verify_ssl)

    def test_cli_overrides_cert_bundle(self):
        environ = {}
        with mock.patch('os.environ', environ):
            parsed_args = FakeParsedArgs(
                verify_ssl=True, ca_bundle='/path/to/cli_bundle.pem'
            )
            config_file_vars = {}
            session_var_map = {'ca_bundle': ('ca_bundle', 'AWS_CA_BUNDLE')}
            session = FakeSession(
                session_vars=session_var_map, config_file_vars=config_file_vars
            )
            globalargs.resolve_verify_ssl(parsed_args, session)
            self.assertEqual(parsed_args.verify_ssl, '/path/to/cli_bundle.pem')

    def test_cli_overrides_env_cert_bundle(self):
        environ = {
            'AWS_CA_BUNDLE': '/path/to/env_bundle.pem',
        }
        with mock.patch('os.environ', environ):
            parsed_args = FakeParsedArgs(
                verify_ssl=True, ca_bundle='/path/to/cli_bundle.pem'
            )
            config_file_vars = {}
            session_var_map = {'ca_bundle': ('ca_bundle', 'AWS_CA_BUNDLE')}
            session = FakeSession(
                session_vars=session_var_map, config_file_vars=config_file_vars
            )
            globalargs.resolve_verify_ssl(parsed_args, session)
            self.assertEqual(parsed_args.verify_ssl, '/path/to/cli_bundle.pem')

    def test_no_verify_ssl_overrides_cli_cert_bundle(self):
        environ = {
            'AWS_CA_BUNDLE': '/path/to/env_bundle.pem',
        }
        with mock.patch('os.environ', environ):
            parsed_args = FakeParsedArgs(
                verify_ssl=False, ca_bundle='/path/to/cli_bundle.pem'
            )
            config_file_vars = {}
            session_var_map = {'ca_bundle': ('ca_bundle', 'AWS_CA_BUNDLE')}
            session = FakeSession(
                session_vars=session_var_map, config_file_vars=config_file_vars
            )
            globalargs.resolve_verify_ssl(parsed_args, session)
            self.assertFalse(parsed_args.verify_ssl)

    def test_no_sign_request_if_option_specified(self):
        args = FakeParsedArgs(sign_request=False)
        session = mock.Mock()
        with mock.patch(
            'awscli.customizations.globalargs._update_default_client_config'
        ) as mock_update:
            globalargs.no_sign_request(args, session)
            mock_update.assert_called_once_with(
                session, 'signature_version', UNSIGNED
            )

    def test_request_signed_by_default(self):
        args = FakeParsedArgs(sign_request=True)
        session = mock.Mock()

        globalargs.no_sign_request(args, session)
        self.assertFalse(session.register.called)

    def test_invalid_endpoint_url(self):
        # Invalid endpoint URL
        parsed_args = FakeParsedArgs(endpoint_url='missing-scheme.com')
        with self.assertRaises(ParamValidationError):
            globalargs.resolve_types(parsed_args)

    def test_valid_endpoint_url(self):
        parsed_args = FakeParsedArgs(endpoint_url='http://custom-endpoint.com')
        globalargs.resolve_types(parsed_args)
        self.assertEqual(
            parsed_args.endpoint_url, 'http://custom-endpoint.com'
        )

    def test_cli_read_timeout(self):
        parsed_args = FakeParsedArgs(read_timeout='60')
        session = get_session()
        globalargs.resolve_cli_read_timeout(parsed_args, session)
        self.assertEqual(parsed_args.read_timeout, 60)
        self.assertEqual(session.get_default_client_config().read_timeout, 60)

    def test_cli_connect_timeout(self):
        parsed_args = FakeParsedArgs(connect_timeout='60')
        session = get_session()
        globalargs.resolve_cli_connect_timeout(parsed_args, session)
        self.assertEqual(parsed_args.connect_timeout, 60)
        self.assertEqual(
            session.get_default_client_config().connect_timeout, 60
        )

    def test_cli_read_timeout_for_blocking(self):
        parsed_args = FakeParsedArgs(read_timeout='0')
        session = get_session()
        globalargs.resolve_cli_read_timeout(parsed_args, session)
        self.assertEqual(parsed_args.read_timeout, None)
        self.assertEqual(
            session.get_default_client_config().read_timeout, None
        )

    def test_cli_connect_timeout_for_blocking(self):
        parsed_args = FakeParsedArgs(connect_timeout='0')
        session = get_session()
        globalargs.resolve_cli_connect_timeout(parsed_args, session)
        self.assertEqual(parsed_args.connect_timeout, None)
        self.assertEqual(
            session.get_default_client_config().connect_timeout, None
        )
